// A Bison parser, made by GNU Bison 3.8.2.

// Skeleton implementation for Bison LALR(1) parsers in C++

// Copyright (C) 2002-2015, 2018-2021 Free Software Foundation, Inc.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// As a special exception, you may create a larger work that contains
// part or all of the Bison parser skeleton and distribute that work
// under terms of your choice, so long as that work isn't itself a
// parser generator using the skeleton or a modified version thereof
// as a parser skeleton.  Alternatively, if you modify or redistribute
// the parser skeleton itself, you may (at your option) remove this
// special exception, which will cause the skeleton and the resulting
// Bison output files to be licensed under the GNU General Public
// License without this special exception.

// This special exception was added by the Free Software Foundation in
// version 2.2 of Bison.

// DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
// especially those whose name start with YY_ or yy_.  They are
// private implementation details that can be changed or removed.





#include "parser.hpp"


// Unqualified %code blocks.
#line 45 "src/asm/parser.y"

	#include <algorithm>
	#include <ctype.h>
	#include <inttypes.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <string_view>

	#include "asm/charmap.hpp"
	#include "asm/fixpoint.hpp"
	#include "asm/format.hpp"
	#include "asm/fstack.hpp"
	#include "asm/main.hpp"
	#include "asm/opt.hpp"
	#include "asm/output.hpp"
	#include "asm/section.hpp"
	#include "asm/symbol.hpp"
	#include "asm/warning.hpp"

	#include "extern/utf8decoder.hpp"

	#include "helpers.hpp"

	using namespace std::literals;

	yy::parser::symbol_type yylex(); // Provided by lexer.cpp

	static uint32_t strToNum(std::vector<int32_t> const &s);
	static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName);
	static size_t strlenUTF8(std::string const &str);
	static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len);
	static size_t charlenUTF8(std::string const &str);
	static std::string charsubUTF8(std::string const &str, uint32_t pos);
	static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName);
	static std::string strrpl(
	    std::string_view str, std::string const &old, std::string const &rep
	);
	static std::string strfmt(
	    std::string const &spec, std::vector<Either<uint32_t, std::string>> const &args
	);
	static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue);
	static void failAssert(AssertionType type);
	static void failAssertMsg(AssertionType type, std::string const &message);

	// The CPU encodes instructions in a logical way, so most instructions actually follow patterns.
	// These enums thus help with bit twiddling to compute opcodes.
	enum { REG_B, REG_C, REG_D, REG_E, REG_H, REG_L, REG_HL_IND, REG_A };

	enum { REG_BC_IND, REG_DE_IND, REG_HL_INDINC, REG_HL_INDDEC };

	// REG_AF == REG_SP since LD/INC/ADD/DEC allow SP, while PUSH/POP allow AF
	enum { REG_BC, REG_DE, REG_HL, REG_SP, REG_AF = REG_SP };

	// CC_NZ == CC_Z ^ 1, and CC_NC == CC_C ^ 1, so `!` can toggle them
	enum { CC_NZ, CC_Z, CC_NC, CC_C };

#line 104 "src/asm/parser.cpp"


#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> // FIXME: INFRINGES ON USER NAME SPACE.
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif


// Whether we are compiled with exception support.
#ifndef YY_EXCEPTIONS
# if defined __GNUC__ && !defined __EXCEPTIONS
#  define YY_EXCEPTIONS 0
# else
#  define YY_EXCEPTIONS 1
# endif
#endif



// Enable debugging if requested.
#if YYDEBUG

// A pseudo ostream that takes yydebug_ into account.
# define YYCDEBUG if (yydebug_) (*yycdebug_)

# define YY_SYMBOL_PRINT(Title, Symbol)         \
  do {                                          \
    if (yydebug_)                               \
    {                                           \
      *yycdebug_ << Title << ' ';               \
      yy_print_ (*yycdebug_, Symbol);           \
      *yycdebug_ << '\n';                       \
    }                                           \
  } while (false)

# define YY_REDUCE_PRINT(Rule)          \
  do {                                  \
    if (yydebug_)                       \
      yy_reduce_print_ (Rule);          \
  } while (false)

# define YY_STACK_PRINT()               \
  do {                                  \
    if (yydebug_)                       \
      yy_stack_print_ ();                \
  } while (false)

#else // !YYDEBUG

# define YYCDEBUG if (false) std::cerr
# define YY_SYMBOL_PRINT(Title, Symbol)  YY_USE (Symbol)
# define YY_REDUCE_PRINT(Rule)           static_cast<void> (0)
# define YY_STACK_PRINT()                static_cast<void> (0)

#endif // !YYDEBUG

#define yyerrok         (yyerrstatus_ = 0)
#define yyclearin       (yyla.clear ())

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab
#define YYRECOVERING()  (!!yyerrstatus_)

namespace yy {
#line 177 "src/asm/parser.cpp"

  /// Build a parser object.
  parser::parser ()
#if YYDEBUG
    : yydebug_ (false),
      yycdebug_ (&std::cerr),
#else
    :
#endif
      yy_lac_established_ (false)
  {}

  parser::~parser ()
  {}

  parser::syntax_error::~syntax_error () YY_NOEXCEPT YY_NOTHROW
  {}

  /*---------.
  | symbol.  |
  `---------*/



  // by_state.
  parser::by_state::by_state () YY_NOEXCEPT
    : state (empty_state)
  {}

  parser::by_state::by_state (const by_state& that) YY_NOEXCEPT
    : state (that.state)
  {}

  void
  parser::by_state::clear () YY_NOEXCEPT
  {
    state = empty_state;
  }

  void
  parser::by_state::move (by_state& that)
  {
    state = that.state;
    that.clear ();
  }

  parser::by_state::by_state (state_type s) YY_NOEXCEPT
    : state (s)
  {}

  parser::symbol_kind_type
  parser::by_state::kind () const YY_NOEXCEPT
  {
    if (state == empty_state)
      return symbol_kind::S_YYEMPTY;
    else
      return YY_CAST (symbol_kind_type, yystos_[+state]);
  }

  parser::stack_symbol_type::stack_symbol_type ()
  {}

  parser::stack_symbol_type::stack_symbol_type (YY_RVREF (stack_symbol_type) that)
    : super_type (YY_MOVE (that.state))
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.YY_MOVE_OR_COPY< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.YY_MOVE_OR_COPY< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.YY_MOVE_OR_COPY< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.YY_MOVE_OR_COPY< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.YY_MOVE_OR_COPY< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.YY_MOVE_OR_COPY< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.YY_MOVE_OR_COPY< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.YY_MOVE_OR_COPY< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.YY_MOVE_OR_COPY< SectionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.YY_MOVE_OR_COPY< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.YY_MOVE_OR_COPY< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.YY_MOVE_OR_COPY< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.YY_MOVE_OR_COPY< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.YY_MOVE_OR_COPY< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.YY_MOVE_OR_COPY< std::vector<int32_t> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.YY_MOVE_OR_COPY< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

#if 201103L <= YY_CPLUSPLUS
    // that is emptied.
    that.state = empty_state;
#endif
  }

  parser::stack_symbol_type::stack_symbol_type (state_type s, YY_MOVE_REF (symbol_type) that)
    : super_type (s)
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.move< SectionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.move< std::vector<int32_t> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

    // that is emptied.
    that.kind_ = symbol_kind::S_YYEMPTY;
  }

#if YY_CPLUSPLUS < 201103L
  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (const stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.copy< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.copy< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.copy< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.copy< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.copy< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.copy< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.copy< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.copy< SectionSpec > (that.value);
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.copy< SectionType > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.copy< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.copy< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.copy< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.copy< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.copy< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.copy< std::vector<int32_t> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.copy< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    return *this;
  }

  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (that.value);
        break;

      case symbol_kind::S_sect_type: // sect_type
        value.move< SectionType > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        value.move< std::vector<int32_t> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    // that is emptied.
    that.state = empty_state;
    return *this;
  }
#endif

  template <typename Base>
  void
  parser::yy_destroy_ (const char* yymsg, basic_symbol<Base>& yysym) const
  {
    if (yymsg)
      YY_SYMBOL_PRINT (yymsg, yysym);
  }

#if YYDEBUG
  template <typename Base>
  void
  parser::yy_print_ (std::ostream& yyo, const basic_symbol<Base>& yysym) const
  {
    std::ostream& yyoutput = yyo;
    YY_USE (yyoutput);
    if (yysym.empty ())
      yyo << "empty symbol";
    else
      {
        symbol_kind_type yykind = yysym.kind ();
        yyo << (yykind < YYNTOKENS ? "token" : "nterm")
            << ' ' << yysym.name () << " (";
        YY_USE (yykind);
        yyo << ')';
      }
  }
#endif

  void
  parser::yypush_ (const char* m, YY_MOVE_REF (stack_symbol_type) sym)
  {
    if (m)
      YY_SYMBOL_PRINT (m, sym);
    yystack_.push (YY_MOVE (sym));
  }

  void
  parser::yypush_ (const char* m, state_type s, YY_MOVE_REF (symbol_type) sym)
  {
#if 201103L <= YY_CPLUSPLUS
    yypush_ (m, stack_symbol_type (s, std::move (sym)));
#else
    stack_symbol_type ss (s, sym);
    yypush_ (m, ss);
#endif
  }

  void
  parser::yypop_ (int n) YY_NOEXCEPT
  {
    yystack_.pop (n);
  }

#if YYDEBUG
  std::ostream&
  parser::debug_stream () const
  {
    return *yycdebug_;
  }

  void
  parser::set_debug_stream (std::ostream& o)
  {
    yycdebug_ = &o;
  }


  parser::debug_level_type
  parser::debug_level () const
  {
    return yydebug_;
  }

  void
  parser::set_debug_level (debug_level_type l)
  {
    yydebug_ = l;
  }
#endif // YYDEBUG

  parser::state_type
  parser::yy_lr_goto_state_ (state_type yystate, int yysym)
  {
    int yyr = yypgoto_[yysym - YYNTOKENS] + yystate;
    if (0 <= yyr && yyr <= yylast_ && yycheck_[yyr] == yystate)
      return yytable_[yyr];
    else
      return yydefgoto_[yysym - YYNTOKENS];
  }

  bool
  parser::yy_pact_value_is_default_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yypact_ninf_;
  }

  bool
  parser::yy_table_value_is_error_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yytable_ninf_;
  }

  int
  parser::operator() ()
  {
    return parse ();
  }

  int
  parser::parse ()
  {
    int yyn;
    /// Length of the RHS of the rule being reduced.
    int yylen = 0;

    // Error handling.
    int yynerrs_ = 0;
    int yyerrstatus_ = 0;

    /// The lookahead symbol.
    symbol_type yyla;

    /// The return value of parse ().
    int yyresult;

    // Discard the LAC context in case there still is one left from a
    // previous invocation.
    yy_lac_discard_ ("init");

#if YY_EXCEPTIONS
    try
#endif // YY_EXCEPTIONS
      {
    YYCDEBUG << "Starting parse\n";


    /* Initialize the stack.  The initial state will be set in
       yynewstate, since the latter expects the semantical and the
       location values to have been already stored, initialize these
       stacks with a primary value.  */
    yystack_.clear ();
    yypush_ (YY_NULLPTR, 0, YY_MOVE (yyla));

  /*-----------------------------------------------.
  | yynewstate -- push a new symbol on the stack.  |
  `-----------------------------------------------*/
  yynewstate:
    YYCDEBUG << "Entering state " << int (yystack_[0].state) << '\n';
    YY_STACK_PRINT ();

    // Accept?
    if (yystack_[0].state == yyfinal_)
      YYACCEPT;

    goto yybackup;


  /*-----------.
  | yybackup.  |
  `-----------*/
  yybackup:
    // Try to take a decision without lookahead.
    yyn = yypact_[+yystack_[0].state];
    if (yy_pact_value_is_default_ (yyn))
      goto yydefault;

    // Read a lookahead token.
    if (yyla.empty ())
      {
        YYCDEBUG << "Reading a token\n";
#if YY_EXCEPTIONS
        try
#endif // YY_EXCEPTIONS
          {
            symbol_type yylookahead (yylex ());
            yyla.move (yylookahead);
          }
#if YY_EXCEPTIONS
        catch (const syntax_error& yyexc)
          {
            YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
            error (yyexc);
            goto yyerrlab1;
          }
#endif // YY_EXCEPTIONS
      }
    YY_SYMBOL_PRINT ("Next token is", yyla);

    if (yyla.kind () == symbol_kind::S_YYerror)
    {
      // The scanner already issued an error message, process directly
      // to error recovery.  But do not keep the error token as
      // lookahead, it is too special and may lead us to an endless
      // loop in error recovery. */
      yyla.kind_ = symbol_kind::S_YYUNDEF;
      goto yyerrlab1;
    }

    /* If the proper action on seeing token YYLA.TYPE is to reduce or
       to detect an error, take that action.  */
    yyn += yyla.kind ();
    if (yyn < 0 || yylast_ < yyn || yycheck_[yyn] != yyla.kind ())
      {
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;
        goto yydefault;
      }

    // Reduce or error.
    yyn = yytable_[yyn];
    if (yyn <= 0)
      {
        if (yy_table_value_is_error_ (yyn))
          goto yyerrlab;
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;

        yyn = -yyn;
        goto yyreduce;
      }

    // Count tokens shifted since error; after three, turn off error status.
    if (yyerrstatus_)
      --yyerrstatus_;

    // Shift the lookahead token.
    yypush_ ("Shifting", state_type (yyn), YY_MOVE (yyla));
    yy_lac_discard_ ("shift");
    goto yynewstate;


  /*-----------------------------------------------------------.
  | yydefault -- do the default action for the current state.  |
  `-----------------------------------------------------------*/
  yydefault:
    yyn = yydefact_[+yystack_[0].state];
    if (yyn == 0)
      goto yyerrlab;
    goto yyreduce;


  /*-----------------------------.
  | yyreduce -- do a reduction.  |
  `-----------------------------*/
  yyreduce:
    yylen = yyr2_[yyn];
    {
      stack_symbol_type yylhs;
      yylhs.state = yy_lr_goto_state_ (yystack_[yylen].state, yyr1_[yyn]);
      /* Variants are always initialized to an empty instance of the
         correct type. The default '$$ = $1' action is NOT applied
         when using variants.  */
      switch (yyr1_[yyn])
    {
      case symbol_kind::S_align_spec: // align_spec
        yylhs.value.emplace< AlignmentSpec > ();
        break;

      case symbol_kind::S_assert_type: // assert_type
        yylhs.value.emplace< AssertionType > ();
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        yylhs.value.emplace< Capture > ();
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        yylhs.value.emplace< Expression > ();
        break;

      case symbol_kind::S_for_args: // for_args
        yylhs.value.emplace< ForArgs > ();
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        yylhs.value.emplace< RPNCommand > ();
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        yylhs.value.emplace< SectionModifier > ();
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        yylhs.value.emplace< SectionSpec > ();
        break;

      case symbol_kind::S_sect_type: // sect_type
        yylhs.value.emplace< SectionType > ();
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        yylhs.value.emplace< StrFmtArgList > ();
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_iconst: // iconst
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_precision_arg: // precision_arg
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        yylhs.value.emplace< int32_t > ();
        break;

      case symbol_kind::S_macro_args: // macro_args
        yylhs.value.emplace< std::shared_ptr<MacroArgs> > ();
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_def_numeric: // def_numeric
      case symbol_kind::S_def_equ: // def_equ
      case symbol_kind::S_redef_equ: // redef_equ
      case symbol_kind::S_def_set: // def_set
      case symbol_kind::S_def_rb: // def_rb
      case symbol_kind::S_def_rw: // def_rw
      case symbol_kind::S_def_rl: // def_rl
      case symbol_kind::S_def_equs: // def_equs
      case symbol_kind::S_redef_equs: // redef_equs
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        yylhs.value.emplace< std::string > ();
        break;

      case symbol_kind::S_ds_args: // ds_args
        yylhs.value.emplace< std::vector<Expression> > ();
        break;

      case symbol_kind::S_charmap_args: // charmap_args
        yylhs.value.emplace< std::vector<int32_t> > ();
        break;

      case symbol_kind::S_purge_args: // purge_args
        yylhs.value.emplace< std::vector<std::string> > ();
        break;

      default:
        break;
    }



      // Perform the reduction.
      YY_REDUCE_PRINT (yyn);
#if YY_EXCEPTIONS
      try
#endif // YY_EXCEPTIONS
        {
          switch (yyn)
            {
  case 5: // $@1: %empty
#line 408 "src/asm/parser.y"
                {
		lexer_SetMode(LEXER_NORMAL);
		lexer_ToggleStringExpansion(true);
	}
#line 1122 "src/asm/parser.cpp"
    break;

  case 6: // lines: error $@1 endofline
#line 411 "src/asm/parser.y"
                    {
		fstk_StopRept();
		yyerrok;
	}
#line 1131 "src/asm/parser.cpp"
    break;

  case 8: // diff_mark: "+"
#line 419 "src/asm/parser.y"
                 {
		::error(
			"syntax error, unexpected + at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1141 "src/asm/parser.cpp"
    break;

  case 9: // diff_mark: "-"
#line 424 "src/asm/parser.y"
                 {
		::error(
			"syntax error, unexpected - at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1151 "src/asm/parser.cpp"
    break;

  case 22: // if: "IF" iconst "end of line"
#line 457 "src/asm/parser.y"
                              {
		lexer_IncIFDepth();

		if (yystack_[1].value.as < int32_t > ())
			lexer_RunIFBlock();
		else
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
	}
#line 1164 "src/asm/parser.cpp"
    break;

  case 23: // elif: "ELIF" iconst "end of line"
#line 468 "src/asm/parser.y"
                                {
		if (lexer_GetIFDepth() == 0)
			fatalerror("Found ELIF outside of an IF construct\n");

		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock())
				fatalerror("Found ELIF after an ELSE block\n");

			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else if (yystack_[1].value.as < int32_t > ()) {
			lexer_RunIFBlock();
		} else {
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
		}
	}
#line 1184 "src/asm/parser.cpp"
    break;

  case 24: // else: "ELSE" "end of line"
#line 486 "src/asm/parser.y"
                         {
		if (lexer_GetIFDepth() == 0)
			fatalerror("Found ELSE outside of an IF construct\n");

		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock())
				fatalerror("Found ELSE after an ELSE block\n");

			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else {
			lexer_RunIFBlock();
			lexer_ReachELSEBlock();
		}
	}
#line 1203 "src/asm/parser.cpp"
    break;

  case 29: // endc: "ENDC"
#line 512 "src/asm/parser.y"
                 {
		lexer_DecIFDepth();
	}
#line 1211 "src/asm/parser.cpp"
    break;

  case 30: // $@2: %empty
#line 518 "src/asm/parser.y"
               {
		lexer_ToggleStringExpansion(false);
	}
#line 1219 "src/asm/parser.cpp"
    break;

  case 31: // def_id: "DEF" $@2 "identifier"
#line 520 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1228 "src/asm/parser.cpp"
    break;

  case 32: // $@3: %empty
#line 527 "src/asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1236 "src/asm/parser.cpp"
    break;

  case 33: // redef_id: "REDEF" $@3 "identifier"
#line 529 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1245 "src/asm/parser.cpp"
    break;

  case 34: // scoped_id: "identifier"
#line 538 "src/asm/parser.y"
           {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1253 "src/asm/parser.cpp"
    break;

  case 35: // scoped_id: "local identifier"
#line 541 "src/asm/parser.y"
                   {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1261 "src/asm/parser.cpp"
    break;

  case 36: // scoped_id: "label"
#line 544 "src/asm/parser.y"
                {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1269 "src/asm/parser.cpp"
    break;

  case 37: // scoped_anon_id: scoped_id
#line 550 "src/asm/parser.y"
                  {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1277 "src/asm/parser.cpp"
    break;

  case 38: // scoped_anon_id: "anonymous label"
#line 553 "src/asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1285 "src/asm/parser.cpp"
    break;

  case 40: // label: ":"
#line 560 "src/asm/parser.y"
                {
		sym_AddAnonLabel();
	}
#line 1293 "src/asm/parser.cpp"
    break;

  case 41: // label: "local identifier"
#line 563 "src/asm/parser.y"
                   {
		sym_AddLocalLabel(yystack_[0].value.as < std::string > ());
	}
#line 1301 "src/asm/parser.cpp"
    break;

  case 42: // label: "local identifier" ":"
#line 566 "src/asm/parser.y"
                         {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
	}
#line 1309 "src/asm/parser.cpp"
    break;

  case 43: // label: "label" ":"
#line 569 "src/asm/parser.y"
                      {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
	}
#line 1317 "src/asm/parser.cpp"
    break;

  case 44: // label: "local identifier" "::"
#line 572 "src/asm/parser.y"
                                {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1326 "src/asm/parser.cpp"
    break;

  case 45: // label: "label" "::"
#line 576 "src/asm/parser.y"
                             {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1335 "src/asm/parser.cpp"
    break;

  case 46: // $@4: %empty
#line 583 "src/asm/parser.y"
           {
		// Parsing 'macro_args' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1344 "src/asm/parser.cpp"
    break;

  case 47: // macro: "identifier" $@4 macro_args
#line 586 "src/asm/parser.y"
                     {
		fstk_RunMacro(yystack_[2].value.as < std::string > (), yystack_[0].value.as < std::shared_ptr<MacroArgs> > ());
	}
#line 1352 "src/asm/parser.cpp"
    break;

  case 48: // macro_args: %empty
#line 592 "src/asm/parser.y"
               {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::make_shared<MacroArgs>();
	}
#line 1360 "src/asm/parser.cpp"
    break;

  case 49: // macro_args: macro_args "string"
#line 595 "src/asm/parser.y"
                            {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::move(yystack_[1].value.as < std::shared_ptr<MacroArgs> > ());
		yylhs.value.as < std::shared_ptr<MacroArgs> > ()->appendArg(std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 1369 "src/asm/parser.cpp"
    break;

  case 89: // def_numeric: def_equ
#line 644 "src/asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1375 "src/asm/parser.cpp"
    break;

  case 90: // def_numeric: redef_equ
#line 645 "src/asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1381 "src/asm/parser.cpp"
    break;

  case 91: // def_numeric: def_set
#line 646 "src/asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1387 "src/asm/parser.cpp"
    break;

  case 92: // def_numeric: def_rb
#line 647 "src/asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1393 "src/asm/parser.cpp"
    break;

  case 93: // def_numeric: def_rw
#line 648 "src/asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1399 "src/asm/parser.cpp"
    break;

  case 94: // def_numeric: def_rl
#line 649 "src/asm/parser.y"
          { yylhs.value.as < std::string > () = yystack_[0].value.as < std::string > (); }
#line 1405 "src/asm/parser.cpp"
    break;

  case 97: // compound_eq: "+="
#line 655 "src/asm/parser.y"
                  {
		yylhs.value.as < RPNCommand > () = RPN_ADD;
	}
#line 1413 "src/asm/parser.cpp"
    break;

  case 98: // compound_eq: "-="
#line 658 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SUB;
	}
#line 1421 "src/asm/parser.cpp"
    break;

  case 99: // compound_eq: "*="
#line 661 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MUL;
	}
#line 1429 "src/asm/parser.cpp"
    break;

  case 100: // compound_eq: "/="
#line 664 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_DIV;
	}
#line 1437 "src/asm/parser.cpp"
    break;

  case 101: // compound_eq: "%="
#line 667 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MOD;
	}
#line 1445 "src/asm/parser.cpp"
    break;

  case 102: // compound_eq: "^="
#line 670 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_XOR;
	}
#line 1453 "src/asm/parser.cpp"
    break;

  case 103: // compound_eq: "|="
#line 673 "src/asm/parser.y"
                   {
		yylhs.value.as < RPNCommand > () = RPN_OR;
	}
#line 1461 "src/asm/parser.cpp"
    break;

  case 104: // compound_eq: "&="
#line 676 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_AND;
	}
#line 1469 "src/asm/parser.cpp"
    break;

  case 105: // compound_eq: "<<="
#line 679 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHL;
	}
#line 1477 "src/asm/parser.cpp"
    break;

  case 106: // compound_eq: ">>="
#line 682 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHR;
	}
#line 1485 "src/asm/parser.cpp"
    break;

  case 107: // align: "ALIGN" align_spec
#line 688 "src/asm/parser.y"
                             {
		sect_AlignPC(yystack_[0].value.as < AlignmentSpec > ().alignment, yystack_[0].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1493 "src/asm/parser.cpp"
    break;

  case 108: // align_spec: uconst
#line 694 "src/asm/parser.y"
               {
		if (yystack_[0].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[0].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[0].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		}
	}
#line 1507 "src/asm/parser.cpp"
    break;

  case 109: // align_spec: uconst "," iconst
#line 703 "src/asm/parser.y"
                              {
		if (yystack_[2].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[2].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else if (yystack_[0].value.as < int32_t > () <= -(1 << yystack_[2].value.as < int32_t > ()) || yystack_[0].value.as < int32_t > () >= 1 << yystack_[2].value.as < int32_t > ()) {
			::error(
				"The absolute alignment offset (%" PRIu32 ") must be less than alignment size (%d)\n",
				static_cast<uint32_t>(yystack_[0].value.as < int32_t > () < 0 ? -yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ()),
				1 << yystack_[2].value.as < int32_t > ()
			);
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[2].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = yystack_[0].value.as < int32_t > () < 0 ? (1 << yystack_[2].value.as < int32_t > ()) + yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ();
		}
	}
#line 1528 "src/asm/parser.cpp"
    break;

  case 110: // $@5: %empty
#line 722 "src/asm/parser.y"
                {
		// Parsing 'opt_list' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1537 "src/asm/parser.cpp"
    break;

  case 114: // opt_list_entry: "string"
#line 734 "src/asm/parser.y"
               {
		opt_Parse(yystack_[0].value.as < std::string > ().c_str());
	}
#line 1545 "src/asm/parser.cpp"
    break;

  case 115: // popo: "POPO"
#line 740 "src/asm/parser.y"
                 {
		opt_Pop();
	}
#line 1553 "src/asm/parser.cpp"
    break;

  case 116: // $@6: %empty
#line 746 "src/asm/parser.y"
                  {
		opt_Push();
		// Parsing 'pusho_opt_list' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1563 "src/asm/parser.cpp"
    break;

  case 118: // pusho_opt_list: %empty
#line 754 "src/asm/parser.y"
               {
		lexer_SetMode(LEXER_NORMAL);
	}
#line 1571 "src/asm/parser.cpp"
    break;

  case 120: // pops: "POPS"
#line 761 "src/asm/parser.y"
                 {
		sect_PopSection();
	}
#line 1579 "src/asm/parser.cpp"
    break;

  case 121: // pushs: "PUSHS"
#line 767 "src/asm/parser.y"
                  {
		sect_PushSection();
	}
#line 1587 "src/asm/parser.cpp"
    break;

  case 122: // endsection: "ENDSECTION"
#line 773 "src/asm/parser.y"
                       {
		sect_EndSection();
	}
#line 1595 "src/asm/parser.cpp"
    break;

  case 123: // fail: "FAIL" string
#line 779 "src/asm/parser.y"
                        {
		fatalerror("%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1603 "src/asm/parser.cpp"
    break;

  case 124: // warn: "WARN" string
#line 785 "src/asm/parser.y"
                        {
		warning(WARNING_USER, "%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1611 "src/asm/parser.cpp"
    break;

  case 125: // assert_type: %empty
#line 791 "src/asm/parser.y"
               {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1619 "src/asm/parser.cpp"
    break;

  case 126: // assert_type: "WARN" ","
#line 794 "src/asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_WARN;
	}
#line 1627 "src/asm/parser.cpp"
    break;

  case 127: // assert_type: "FAIL" ","
#line 797 "src/asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1635 "src/asm/parser.cpp"
    break;

  case 128: // assert_type: "FATAL" ","
#line 800 "src/asm/parser.y"
                          {
		yylhs.value.as < AssertionType > () = ASSERT_FATAL;
	}
#line 1643 "src/asm/parser.cpp"
    break;

  case 129: // assert: "ASSERT" assert_type relocexpr
#line 806 "src/asm/parser.y"
                                         {
		if (!yystack_[0].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[1].value.as < AssertionType > (), yystack_[0].value.as < Expression > (), "", sect_GetOutputOffset());
		} else if (yystack_[0].value.as < Expression > ().value() == 0) {
			failAssert(yystack_[1].value.as < AssertionType > ());
		}
	}
#line 1655 "src/asm/parser.cpp"
    break;

  case 130: // assert: "ASSERT" assert_type relocexpr "," string
#line 813 "src/asm/parser.y"
                                                        {
		if (!yystack_[2].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[3].value.as < AssertionType > (), yystack_[2].value.as < Expression > (), yystack_[0].value.as < std::string > (), sect_GetOutputOffset());
		} else if (yystack_[2].value.as < Expression > ().value() == 0) {
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
		}
	}
#line 1667 "src/asm/parser.cpp"
    break;

  case 131: // assert: "STATIC_ASSERT" assert_type iconst
#line 820 "src/asm/parser.y"
                                               {
		if (yystack_[0].value.as < int32_t > () == 0)
			failAssert(yystack_[1].value.as < AssertionType > ());
	}
#line 1676 "src/asm/parser.cpp"
    break;

  case 132: // assert: "STATIC_ASSERT" assert_type iconst "," string
#line 824 "src/asm/parser.y"
                                                            {
		if (yystack_[2].value.as < int32_t > () == 0)
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
	}
#line 1685 "src/asm/parser.cpp"
    break;

  case 133: // shift: "SHIFT" shift_const
#line 831 "src/asm/parser.y"
                              {
		if (MacroArgs *macroArgs = fstk_GetCurrentMacroArgs(); macroArgs) {
			macroArgs->shiftArgs(yystack_[0].value.as < int32_t > ());
		} else {
			::error("Cannot shift macro arguments outside of a macro\n");
		}
	}
#line 1697 "src/asm/parser.cpp"
    break;

  case 134: // shift_const: %empty
#line 841 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1705 "src/asm/parser.cpp"
    break;

  case 135: // shift_const: iconst
#line 844 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1711 "src/asm/parser.cpp"
    break;

  case 136: // load: "LOAD" sect_mod string "," sect_type sect_org sect_attrs
#line 848 "src/asm/parser.y"
                                                                     {
		sect_SetLoadSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 1719 "src/asm/parser.cpp"
    break;

  case 137: // load: "ENDL"
#line 851 "src/asm/parser.y"
                   {
		sect_EndLoadSection(nullptr);
	}
#line 1727 "src/asm/parser.cpp"
    break;

  case 138: // rept: "REPT" uconst "end of line" capture_rept endofline
#line 857 "src/asm/parser.y"
                                                       {
		if (yystack_[1].value.as < Capture > ().span.ptr)
			fstk_RunRept(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
	}
#line 1736 "src/asm/parser.cpp"
    break;

  case 139: // $@7: %empty
#line 864 "src/asm/parser.y"
                {
		lexer_ToggleStringExpansion(false);
	}
#line 1744 "src/asm/parser.cpp"
    break;

  case 140: // $@8: %empty
#line 866 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
	}
#line 1752 "src/asm/parser.cpp"
    break;

  case 141: // for: "FOR" $@7 "identifier" $@8 "," for_args "end of line" capture_rept endofline
#line 868 "src/asm/parser.y"
                                                        {
		if (yystack_[1].value.as < Capture > ().span.ptr)
			fstk_RunFor(yystack_[6].value.as < std::string > (), yystack_[3].value.as < ForArgs > ().start, yystack_[3].value.as < ForArgs > ().stop, yystack_[3].value.as < ForArgs > ().step, yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
	}
#line 1761 "src/asm/parser.cpp"
    break;

  case 142: // capture_rept: %empty
#line 875 "src/asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureRept();
	}
#line 1769 "src/asm/parser.cpp"
    break;

  case 143: // for_args: iconst
#line 881 "src/asm/parser.y"
               {
		yylhs.value.as < ForArgs > ().start = 0;
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1779 "src/asm/parser.cpp"
    break;

  case 144: // for_args: iconst "," iconst
#line 886 "src/asm/parser.y"
                              {
		yylhs.value.as < ForArgs > ().start = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1789 "src/asm/parser.cpp"
    break;

  case 145: // for_args: iconst "," iconst "," iconst
#line 891 "src/asm/parser.y"
                                           {
		yylhs.value.as < ForArgs > ().start = yystack_[4].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = yystack_[0].value.as < int32_t > ();
	}
#line 1799 "src/asm/parser.cpp"
    break;

  case 146: // break: label "BREAK" endofline
#line 899 "src/asm/parser.y"
                                  {
		if (fstk_Break())
			lexer_SetMode(LEXER_SKIP_TO_ENDR);
	}
#line 1808 "src/asm/parser.cpp"
    break;

  case 147: // $@9: %empty
#line 906 "src/asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1816 "src/asm/parser.cpp"
    break;

  case 148: // $@10: %empty
#line 908 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
	}
#line 1824 "src/asm/parser.cpp"
    break;

  case 149: // def_macro: "MACRO" $@9 "identifier" $@10 "end of line" capture_macro endofline
#line 910 "src/asm/parser.y"
                                          {
		if (yystack_[1].value.as < Capture > ().span.ptr)
			sym_AddMacro(yystack_[4].value.as < std::string > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
	}
#line 1833 "src/asm/parser.cpp"
    break;

  case 150: // capture_macro: %empty
#line 917 "src/asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureMacro();
	}
#line 1841 "src/asm/parser.cpp"
    break;

  case 151: // rsset: "RSSET" uconst
#line 923 "src/asm/parser.y"
                         {
		sym_SetRSValue(yystack_[0].value.as < int32_t > ());
	}
#line 1849 "src/asm/parser.cpp"
    break;

  case 152: // rsreset: "RSRESET"
#line 929 "src/asm/parser.y"
                    {
		sym_SetRSValue(0);
	}
#line 1857 "src/asm/parser.cpp"
    break;

  case 153: // rs_uconst: %empty
#line 935 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1865 "src/asm/parser.cpp"
    break;

  case 154: // rs_uconst: uconst
#line 938 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1871 "src/asm/parser.cpp"
    break;

  case 155: // union: "UNION"
#line 942 "src/asm/parser.y"
                  {
		sect_StartUnion();
	}
#line 1879 "src/asm/parser.cpp"
    break;

  case 156: // nextu: "NEXTU"
#line 948 "src/asm/parser.y"
                  {
		sect_NextUnionMember();
	}
#line 1887 "src/asm/parser.cpp"
    break;

  case 157: // endu: "ENDU"
#line 954 "src/asm/parser.y"
                 {
		sect_EndUnion();
	}
#line 1895 "src/asm/parser.cpp"
    break;

  case 158: // ds: "DS" uconst
#line 960 "src/asm/parser.y"
                      {
		sect_Skip(yystack_[0].value.as < int32_t > (), true);
	}
#line 1903 "src/asm/parser.cpp"
    break;

  case 159: // ds: "DS" uconst "," ds_args trailing_comma
#line 963 "src/asm/parser.y"
                                                     {
		sect_RelBytes(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < std::vector<Expression> > ());
	}
#line 1911 "src/asm/parser.cpp"
    break;

  case 160: // ds: "DS" "ALIGN" "[" align_spec "]" trailing_comma
#line 966 "src/asm/parser.y"
                                                                   {
		uint32_t n = sect_GetAlignBytes(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);

		sect_Skip(n, true);
		sect_AlignPC(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1922 "src/asm/parser.cpp"
    break;

  case 161: // ds: "DS" "ALIGN" "[" align_spec "]" "," ds_args trailing_comma
#line 972 "src/asm/parser.y"
                                                                                 {
		uint32_t n = sect_GetAlignBytes(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);

		sect_RelBytes(n, yystack_[1].value.as < std::vector<Expression> > ());
		sect_AlignPC(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1933 "src/asm/parser.cpp"
    break;

  case 162: // ds_args: reloc_8bit
#line 981 "src/asm/parser.y"
                   {
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1941 "src/asm/parser.cpp"
    break;

  case 163: // ds_args: ds_args "," reloc_8bit
#line 984 "src/asm/parser.y"
                                   {
		yylhs.value.as < std::vector<Expression> > () = std::move(yystack_[2].value.as < std::vector<Expression> > ());
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1950 "src/asm/parser.cpp"
    break;

  case 164: // db: "DB"
#line 991 "src/asm/parser.y"
               {
		sect_Skip(1, false);
	}
#line 1958 "src/asm/parser.cpp"
    break;

  case 166: // dw: "DW"
#line 998 "src/asm/parser.y"
               {
		sect_Skip(2, false);
	}
#line 1966 "src/asm/parser.cpp"
    break;

  case 168: // dl: "DL"
#line 1005 "src/asm/parser.y"
               {
		sect_Skip(4, false);
	}
#line 1974 "src/asm/parser.cpp"
    break;

  case 170: // def_equ: def_id "EQU" iconst
#line 1012 "src/asm/parser.y"
                              {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddEqu(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1983 "src/asm/parser.cpp"
    break;

  case 171: // redef_equ: redef_id "EQU" iconst
#line 1019 "src/asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_RedefEqu(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1992 "src/asm/parser.cpp"
    break;

  case 172: // def_set: def_id "=" iconst
#line 1026 "src/asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddVar(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 2001 "src/asm/parser.cpp"
    break;

  case 173: // def_set: redef_id "=" iconst
#line 1030 "src/asm/parser.y"
                                    {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddVar(yylhs.value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 2010 "src/asm/parser.cpp"
    break;

  case 174: // def_set: def_id compound_eq iconst
#line 1034 "src/asm/parser.y"
                                    {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		compoundAssignment(yylhs.value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 2019 "src/asm/parser.cpp"
    break;

  case 175: // def_set: redef_id compound_eq iconst
#line 1038 "src/asm/parser.y"
                                      {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		compoundAssignment(yylhs.value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 2028 "src/asm/parser.cpp"
    break;

  case 176: // def_rb: def_id "RB" rs_uconst
#line 1045 "src/asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + yystack_[0].value.as < int32_t > ());
	}
#line 2039 "src/asm/parser.cpp"
    break;

  case 177: // def_rw: def_id "RW" rs_uconst
#line 1054 "src/asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + 2 * yystack_[0].value.as < int32_t > ());
	}
#line 2050 "src/asm/parser.cpp"
    break;

  case 178: // def_rl: def_id "rl" rs_uconst
#line 1063 "src/asm/parser.y"
                                {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yylhs.value.as < std::string > (), rs);
		sym_SetRSValue(rs + 4 * yystack_[0].value.as < int32_t > ());
	}
#line 2061 "src/asm/parser.cpp"
    break;

  case 179: // def_equs: def_id "EQUS" string
#line 1072 "src/asm/parser.y"
                               {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_AddString(yylhs.value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 2070 "src/asm/parser.cpp"
    break;

  case 180: // redef_equs: redef_id "EQUS" string
#line 1079 "src/asm/parser.y"
                                 {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		sym_RedefString(yylhs.value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 2079 "src/asm/parser.cpp"
    break;

  case 181: // $@11: %empty
#line 1086 "src/asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 2087 "src/asm/parser.cpp"
    break;

  case 182: // purge: "PURGE" $@11 purge_args trailing_comma
#line 1088 "src/asm/parser.y"
                                    {
		for (std::string &arg : yystack_[1].value.as < std::vector<std::string> > ())
			sym_Purge(arg);
		lexer_ToggleStringExpansion(true);
	}
#line 2097 "src/asm/parser.cpp"
    break;

  case 183: // purge_args: scoped_id
#line 1096 "src/asm/parser.y"
                  {
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2105 "src/asm/parser.cpp"
    break;

  case 184: // purge_args: purge_args "," scoped_id
#line 1099 "src/asm/parser.y"
                                     {
		yylhs.value.as < std::vector<std::string> > () = std::move(yystack_[2].value.as < std::vector<std::string> > ());
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2114 "src/asm/parser.cpp"
    break;

  case 188: // export_list_entry: scoped_id
#line 1113 "src/asm/parser.y"
                  {
		sym_Export(yystack_[0].value.as < std::string > ());
	}
#line 2122 "src/asm/parser.cpp"
    break;

  case 189: // export_def: "EXPORT" def_numeric
#line 1119 "src/asm/parser.y"
                               {
		sym_Export(yystack_[0].value.as < std::string > ());
	}
#line 2130 "src/asm/parser.cpp"
    break;

  case 190: // include: label "INCLUDE" string endofline
#line 1125 "src/asm/parser.y"
                                           {
		fstk_RunInclude(yystack_[1].value.as < std::string > (), false);
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2140 "src/asm/parser.cpp"
    break;

  case 191: // incbin: "INCBIN" string
#line 1133 "src/asm/parser.y"
                          {
		sect_BinaryFile(yystack_[0].value.as < std::string > (), 0);
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2150 "src/asm/parser.cpp"
    break;

  case 192: // incbin: "INCBIN" string "," iconst
#line 1138 "src/asm/parser.y"
                                         {
		sect_BinaryFile(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2160 "src/asm/parser.cpp"
    break;

  case 193: // incbin: "INCBIN" string "," iconst "," iconst
#line 1143 "src/asm/parser.y"
                                                      {
		sect_BinaryFileSlice(yystack_[4].value.as < std::string > (), yystack_[2].value.as < int32_t > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2170 "src/asm/parser.cpp"
    break;

  case 194: // charmap: "CHARMAP" string "," charmap_args trailing_comma
#line 1151 "src/asm/parser.y"
                                                             {
		charmap_Add(yystack_[3].value.as < std::string > (), std::move(yystack_[1].value.as < std::vector<int32_t> > ()));
	}
#line 2178 "src/asm/parser.cpp"
    break;

  case 195: // charmap_args: iconst
#line 1157 "src/asm/parser.y"
               {
		yylhs.value.as < std::vector<int32_t> > ().push_back(std::move(yystack_[0].value.as < int32_t > ()));
	}
#line 2186 "src/asm/parser.cpp"
    break;

  case 196: // charmap_args: charmap_args "," iconst
#line 1160 "src/asm/parser.y"
                                    {
		yylhs.value.as < std::vector<int32_t> > () = std::move(yystack_[2].value.as < std::vector<int32_t> > ());
		yylhs.value.as < std::vector<int32_t> > ().push_back(std::move(yystack_[0].value.as < int32_t > ()));
	}
#line 2195 "src/asm/parser.cpp"
    break;

  case 197: // newcharmap: "NEWCHARMAP" "identifier"
#line 1167 "src/asm/parser.y"
                          {
		charmap_New(yystack_[0].value.as < std::string > (), nullptr);
	}
#line 2203 "src/asm/parser.cpp"
    break;

  case 198: // newcharmap: "NEWCHARMAP" "identifier" "," "identifier"
#line 1170 "src/asm/parser.y"
                                     {
		charmap_New(yystack_[2].value.as < std::string > (), &yystack_[0].value.as < std::string > ());
	}
#line 2211 "src/asm/parser.cpp"
    break;

  case 199: // setcharmap: "SETCHARMAP" "identifier"
#line 1176 "src/asm/parser.y"
                          {
		charmap_Set(yystack_[0].value.as < std::string > ());
	}
#line 2219 "src/asm/parser.cpp"
    break;

  case 200: // pushc: "PUSHC"
#line 1182 "src/asm/parser.y"
                  {
		charmap_Push();
	}
#line 2227 "src/asm/parser.cpp"
    break;

  case 201: // pushc_setcharmap: "PUSHC" "identifier"
#line 1188 "src/asm/parser.y"
                     {
		charmap_Push();
		charmap_Set(yystack_[0].value.as < std::string > ());
	}
#line 2236 "src/asm/parser.cpp"
    break;

  case 202: // popc: "POPC"
#line 1195 "src/asm/parser.y"
                 {
		charmap_Pop();
	}
#line 2244 "src/asm/parser.cpp"
    break;

  case 204: // println: "PRINTLN"
#line 1203 "src/asm/parser.y"
                    {
		putchar('\n');
		fflush(stdout);
	}
#line 2253 "src/asm/parser.cpp"
    break;

  case 205: // println: "PRINTLN" print_exprs trailing_comma
#line 1207 "src/asm/parser.y"
                                                 {
		putchar('\n');
		fflush(stdout);
	}
#line 2262 "src/asm/parser.cpp"
    break;

  case 208: // print_expr: const_no_str
#line 1219 "src/asm/parser.y"
                     {
		printf("$%" PRIX32, yystack_[0].value.as < int32_t > ());
	}
#line 2270 "src/asm/parser.cpp"
    break;

  case 209: // print_expr: string
#line 1222 "src/asm/parser.y"
                 {
		// Allow printing NUL characters
		fwrite(yystack_[0].value.as < std::string > ().data(), 1, yystack_[0].value.as < std::string > ().length(), stdout);
	}
#line 2279 "src/asm/parser.cpp"
    break;

  case 210: // bit_const: iconst
#line 1229 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0 || yylhs.value.as < int32_t > () > 7) {
			::error("Bit number must be between 0 and 7, not %" PRId32 "\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = 0;
		}
	}
#line 2291 "src/asm/parser.cpp"
    break;

  case 213: // constlist_8bit_entry: reloc_8bit_no_str
#line 1244 "src/asm/parser.y"
                          {
		sect_RelByte(yystack_[0].value.as < Expression > (), 0);
	}
#line 2299 "src/asm/parser.cpp"
    break;

  case 214: // constlist_8bit_entry: string
#line 1247 "src/asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_ByteString(output);
	}
#line 2308 "src/asm/parser.cpp"
    break;

  case 217: // constlist_16bit_entry: reloc_16bit_no_str
#line 1259 "src/asm/parser.y"
                           {
		sect_RelWord(yystack_[0].value.as < Expression > (), 0);
	}
#line 2316 "src/asm/parser.cpp"
    break;

  case 218: // constlist_16bit_entry: string
#line 1262 "src/asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_WordString(output);
	}
#line 2325 "src/asm/parser.cpp"
    break;

  case 221: // constlist_32bit_entry: relocexpr_no_str
#line 1274 "src/asm/parser.y"
                         {
		sect_RelLong(yystack_[0].value.as < Expression > (), 0);
	}
#line 2333 "src/asm/parser.cpp"
    break;

  case 222: // constlist_32bit_entry: string
#line 1277 "src/asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		sect_LongString(output);
	}
#line 2342 "src/asm/parser.cpp"
    break;

  case 223: // reloc_8bit: relocexpr
#line 1284 "src/asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2351 "src/asm/parser.cpp"
    break;

  case 224: // reloc_8bit_no_str: relocexpr_no_str
#line 1291 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2360 "src/asm/parser.cpp"
    break;

  case 225: // reloc_8bit_offset: "+" relocexpr
#line 1298 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2369 "src/asm/parser.cpp"
    break;

  case 226: // reloc_8bit_offset: "-" relocexpr
#line 1302 "src/asm/parser.y"
                           {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NEG, std::move(yystack_[0].value.as < Expression > ()));
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2378 "src/asm/parser.cpp"
    break;

  case 227: // reloc_16bit: relocexpr
#line 1309 "src/asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(16);
	}
#line 2387 "src/asm/parser.cpp"
    break;

  case 228: // reloc_16bit_no_str: relocexpr_no_str
#line 1316 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(16);
	}
#line 2396 "src/asm/parser.cpp"
    break;

  case 229: // relocexpr: relocexpr_no_str
#line 1323 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2404 "src/asm/parser.cpp"
    break;

  case 230: // relocexpr: string
#line 1326 "src/asm/parser.y"
                 {
		std::vector<int32_t> output = charmap_Convert(yystack_[0].value.as < std::string > ());
		yylhs.value.as < Expression > ().makeNumber(strToNum(output));
	}
#line 2413 "src/asm/parser.cpp"
    break;

  case 231: // relocexpr_no_str: scoped_anon_id
#line 1333 "src/asm/parser.y"
                       {
		yylhs.value.as < Expression > ().makeSymbol(yystack_[0].value.as < std::string > ());
	}
#line 2421 "src/asm/parser.cpp"
    break;

  case 232: // relocexpr_no_str: "number"
#line 1336 "src/asm/parser.y"
                 {
		yylhs.value.as < Expression > ().makeNumber(yystack_[0].value.as < int32_t > ());
	}
#line 2429 "src/asm/parser.cpp"
    break;

  case 233: // relocexpr_no_str: "!" relocexpr
#line 1339 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_LOGNOT, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2437 "src/asm/parser.cpp"
    break;

  case 234: // relocexpr_no_str: relocexpr "||" relocexpr
#line 1342 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2445 "src/asm/parser.cpp"
    break;

  case 235: // relocexpr_no_str: relocexpr "&&" relocexpr
#line 1345 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGAND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2453 "src/asm/parser.cpp"
    break;

  case 236: // relocexpr_no_str: relocexpr "==" relocexpr
#line 1348 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGEQ, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2461 "src/asm/parser.cpp"
    break;

  case 237: // relocexpr_no_str: relocexpr ">" relocexpr
#line 1351 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2469 "src/asm/parser.cpp"
    break;

  case 238: // relocexpr_no_str: relocexpr "<" relocexpr
#line 1354 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2477 "src/asm/parser.cpp"
    break;

  case 239: // relocexpr_no_str: relocexpr ">=" relocexpr
#line 1357 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2485 "src/asm/parser.cpp"
    break;

  case 240: // relocexpr_no_str: relocexpr "<=" relocexpr
#line 1360 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2493 "src/asm/parser.cpp"
    break;

  case 241: // relocexpr_no_str: relocexpr "!=" relocexpr
#line 1363 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGNE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2501 "src/asm/parser.cpp"
    break;

  case 242: // relocexpr_no_str: relocexpr "+" relocexpr
#line 1366 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_ADD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2509 "src/asm/parser.cpp"
    break;

  case 243: // relocexpr_no_str: relocexpr "-" relocexpr
#line 1369 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SUB, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2517 "src/asm/parser.cpp"
    break;

  case 244: // relocexpr_no_str: relocexpr "^" relocexpr
#line 1372 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_XOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2525 "src/asm/parser.cpp"
    break;

  case 245: // relocexpr_no_str: relocexpr "|" relocexpr
#line 1375 "src/asm/parser.y"
                                    {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_OR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2533 "src/asm/parser.cpp"
    break;

  case 246: // relocexpr_no_str: relocexpr "&" relocexpr
#line 1378 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_AND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2541 "src/asm/parser.cpp"
    break;

  case 247: // relocexpr_no_str: relocexpr "<<" relocexpr
#line 1381 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2549 "src/asm/parser.cpp"
    break;

  case 248: // relocexpr_no_str: relocexpr ">>" relocexpr
#line 1384 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2557 "src/asm/parser.cpp"
    break;

  case 249: // relocexpr_no_str: relocexpr ">>>" relocexpr
#line 1387 "src/asm/parser.y"
                                      {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_USHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2565 "src/asm/parser.cpp"
    break;

  case 250: // relocexpr_no_str: relocexpr "*" relocexpr
#line 1390 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MUL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2573 "src/asm/parser.cpp"
    break;

  case 251: // relocexpr_no_str: relocexpr "/" relocexpr
#line 1393 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_DIV, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2581 "src/asm/parser.cpp"
    break;

  case 252: // relocexpr_no_str: relocexpr "%" relocexpr
#line 1396 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MOD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2589 "src/asm/parser.cpp"
    break;

  case 253: // relocexpr_no_str: relocexpr "**" relocexpr
#line 1399 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_EXP, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2597 "src/asm/parser.cpp"
    break;

  case 254: // relocexpr_no_str: "+" relocexpr
#line 1402 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2605 "src/asm/parser.cpp"
    break;

  case 255: // relocexpr_no_str: "-" relocexpr
#line 1405 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NEG, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2613 "src/asm/parser.cpp"
    break;

  case 256: // relocexpr_no_str: "~" relocexpr
#line 1408 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_NOT, std::move(yystack_[0].value.as < Expression > ()));
	}
#line 2621 "src/asm/parser.cpp"
    break;

  case 257: // relocexpr_no_str: "HIGH" "(" relocexpr ")"
#line 1411 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_HIGH, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2629 "src/asm/parser.cpp"
    break;

  case 258: // relocexpr_no_str: "LOW" "(" relocexpr ")"
#line 1414 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_LOW, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2637 "src/asm/parser.cpp"
    break;

  case 259: // relocexpr_no_str: "BITWIDTH" "(" relocexpr ")"
#line 1417 "src/asm/parser.y"
                                              {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_BITWIDTH, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2645 "src/asm/parser.cpp"
    break;

  case 260: // relocexpr_no_str: "TZCOUNT" "(" relocexpr ")"
#line 1420 "src/asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeUnaryOp(RPN_TZCOUNT, std::move(yystack_[1].value.as < Expression > ()));
	}
#line 2653 "src/asm/parser.cpp"
    break;

  case 261: // relocexpr_no_str: "ISCONST" "(" relocexpr ")"
#line 1423 "src/asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeNumber(yystack_[1].value.as < Expression > ().isKnown());
	}
#line 2661 "src/asm/parser.cpp"
    break;

  case 262: // relocexpr_no_str: "BANK" "(" scoped_anon_id ")"
#line 1426 "src/asm/parser.y"
                                               {
		// '@' is also an ID; it is handled here
		yylhs.value.as < Expression > ().makeBankSymbol(yystack_[1].value.as < std::string > ());
	}
#line 2670 "src/asm/parser.cpp"
    break;

  case 263: // relocexpr_no_str: "BANK" "(" string ")"
#line 1430 "src/asm/parser.y"
                                       {
		yylhs.value.as < Expression > ().makeBankSection(yystack_[1].value.as < std::string > ());
	}
#line 2678 "src/asm/parser.cpp"
    break;

  case 264: // relocexpr_no_str: "SIZEOF" "(" string ")"
#line 1433 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeSizeOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2686 "src/asm/parser.cpp"
    break;

  case 265: // relocexpr_no_str: "STARTOF" "(" string ")"
#line 1436 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeStartOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2694 "src/asm/parser.cpp"
    break;

  case 266: // relocexpr_no_str: "SIZEOF" "(" sect_type ")"
#line 1439 "src/asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeSizeOfSectionType(yystack_[1].value.as < SectionType > ());
	}
#line 2702 "src/asm/parser.cpp"
    break;

  case 267: // relocexpr_no_str: "STARTOF" "(" sect_type ")"
#line 1442 "src/asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeStartOfSectionType(yystack_[1].value.as < SectionType > ());
	}
#line 2710 "src/asm/parser.cpp"
    break;

  case 268: // $@12: %empty
#line 1445 "src/asm/parser.y"
                 {
		lexer_ToggleStringExpansion(false);
	}
#line 2718 "src/asm/parser.cpp"
    break;

  case 269: // relocexpr_no_str: "DEF" $@12 "(" scoped_anon_id ")"
#line 1447 "src/asm/parser.y"
                                       {
		yylhs.value.as < Expression > ().makeNumber(sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ()) != nullptr);
		lexer_ToggleStringExpansion(true);
	}
#line 2727 "src/asm/parser.cpp"
    break;

  case 270: // relocexpr_no_str: "ROUND" "(" iconst precision_arg ")"
#line 1451 "src/asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(fix_Round(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2735 "src/asm/parser.cpp"
    break;

  case 271: // relocexpr_no_str: "CEIL" "(" iconst precision_arg ")"
#line 1454 "src/asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_Ceil(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2743 "src/asm/parser.cpp"
    break;

  case 272: // relocexpr_no_str: "FLOOR" "(" iconst precision_arg ")"
#line 1457 "src/asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(fix_Floor(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2751 "src/asm/parser.cpp"
    break;

  case 273: // relocexpr_no_str: "FDIV" "(" iconst "," iconst precision_arg ")"
#line 1460 "src/asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Div(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2759 "src/asm/parser.cpp"
    break;

  case 274: // relocexpr_no_str: "FMUL" "(" iconst "," iconst precision_arg ")"
#line 1463 "src/asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Mul(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2767 "src/asm/parser.cpp"
    break;

  case 275: // relocexpr_no_str: "FMOD" "(" iconst "," iconst precision_arg ")"
#line 1466 "src/asm/parser.y"
                                                                  {
		yylhs.value.as < Expression > ().makeNumber(fix_Mod(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2775 "src/asm/parser.cpp"
    break;

  case 276: // relocexpr_no_str: "POW" "(" iconst "," iconst precision_arg ")"
#line 1469 "src/asm/parser.y"
                                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Pow(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2783 "src/asm/parser.cpp"
    break;

  case 277: // relocexpr_no_str: "LOG" "(" iconst "," iconst precision_arg ")"
#line 1472 "src/asm/parser.y"
                                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Log(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2791 "src/asm/parser.cpp"
    break;

  case 278: // relocexpr_no_str: "SIN" "(" iconst precision_arg ")"
#line 1475 "src/asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Sin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2799 "src/asm/parser.cpp"
    break;

  case 279: // relocexpr_no_str: "COS" "(" iconst precision_arg ")"
#line 1478 "src/asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Cos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2807 "src/asm/parser.cpp"
    break;

  case 280: // relocexpr_no_str: "TAN" "(" iconst precision_arg ")"
#line 1481 "src/asm/parser.y"
                                                    {
		yylhs.value.as < Expression > ().makeNumber(fix_Tan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2815 "src/asm/parser.cpp"
    break;

  case 281: // relocexpr_no_str: "ASIN" "(" iconst precision_arg ")"
#line 1484 "src/asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ASin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2823 "src/asm/parser.cpp"
    break;

  case 282: // relocexpr_no_str: "ACOS" "(" iconst precision_arg ")"
#line 1487 "src/asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ACos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2831 "src/asm/parser.cpp"
    break;

  case 283: // relocexpr_no_str: "ATAN" "(" iconst precision_arg ")"
#line 1490 "src/asm/parser.y"
                                                     {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2839 "src/asm/parser.cpp"
    break;

  case 284: // relocexpr_no_str: "ATAN2" "(" iconst "," iconst precision_arg ")"
#line 1493 "src/asm/parser.y"
                                                                   {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan2(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2847 "src/asm/parser.cpp"
    break;

  case 285: // relocexpr_no_str: "STRCMP" "(" string "," string ")"
#line 1496 "src/asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(yystack_[3].value.as < std::string > ().compare(yystack_[1].value.as < std::string > ()));
	}
#line 2855 "src/asm/parser.cpp"
    break;

  case 286: // relocexpr_no_str: "STRIN" "(" string "," string ")"
#line 1499 "src/asm/parser.y"
                                                     {
		auto pos = yystack_[3].value.as < std::string > ().find(yystack_[1].value.as < std::string > ());

		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2865 "src/asm/parser.cpp"
    break;

  case 287: // relocexpr_no_str: "STRRIN" "(" string "," string ")"
#line 1504 "src/asm/parser.y"
                                                      {
		auto pos = yystack_[3].value.as < std::string > ().rfind(yystack_[1].value.as < std::string > ());

		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2875 "src/asm/parser.cpp"
    break;

  case 288: // relocexpr_no_str: "STRLEN" "(" string ")"
#line 1509 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeNumber(strlenUTF8(yystack_[1].value.as < std::string > ()));
	}
#line 2883 "src/asm/parser.cpp"
    break;

  case 289: // relocexpr_no_str: "CHARLEN" "(" string ")"
#line 1512 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeNumber(charlenUTF8(yystack_[1].value.as < std::string > ()));
	}
#line 2891 "src/asm/parser.cpp"
    break;

  case 290: // relocexpr_no_str: "INCHARMAP" "(" string ")"
#line 1515 "src/asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeNumber(charmap_HasChar(yystack_[1].value.as < std::string > ()));
	}
#line 2899 "src/asm/parser.cpp"
    break;

  case 291: // relocexpr_no_str: "(" relocexpr ")"
#line 1518 "src/asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 2907 "src/asm/parser.cpp"
    break;

  case 292: // uconst: iconst
#line 1524 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0)
			fatalerror("Constant must not be negative: %d\n", yylhs.value.as < int32_t > ());
	}
#line 2917 "src/asm/parser.cpp"
    break;

  case 293: // iconst: relocexpr
#line 1532 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 2925 "src/asm/parser.cpp"
    break;

  case 294: // const_no_str: relocexpr_no_str
#line 1538 "src/asm/parser.y"
                         {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 2933 "src/asm/parser.cpp"
    break;

  case 295: // precision_arg: %empty
#line 1544 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = fix_Precision();
	}
#line 2941 "src/asm/parser.cpp"
    break;

  case 296: // precision_arg: "," iconst
#line 1547 "src/asm/parser.y"
                       {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 1 || yylhs.value.as < int32_t > () > 31) {
			::error("Fixed-point precision must be between 1 and 31, not %" PRId32 "\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = fix_Precision();
		}
	}
#line 2953 "src/asm/parser.cpp"
    break;

  case 297: // string: "string"
#line 1557 "src/asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 2961 "src/asm/parser.cpp"
    break;

  case 298: // string: "STRSUB" "(" string "," iconst "," uconst ")"
#line 1560 "src/asm/parser.y"
                                                                   {
		size_t len = strlenUTF8(yystack_[5].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[3].value.as < int32_t > (), len, "STRSUB");

		yylhs.value.as < std::string > () = strsubUTF8(yystack_[5].value.as < std::string > (), pos, yystack_[1].value.as < int32_t > ());
	}
#line 2972 "src/asm/parser.cpp"
    break;

  case 299: // string: "STRSUB" "(" string "," iconst ")"
#line 1566 "src/asm/parser.y"
                                                      {
		size_t len = strlenUTF8(yystack_[3].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "STRSUB");

		yylhs.value.as < std::string > () = strsubUTF8(yystack_[3].value.as < std::string > (), pos, pos > len ? 0 : len + 1 - pos);
	}
#line 2983 "src/asm/parser.cpp"
    break;

  case 300: // string: "CHARSUB" "(" string "," iconst ")"
#line 1572 "src/asm/parser.y"
                                                       {
		size_t len = charlenUTF8(yystack_[3].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "CHARSUB");

		yylhs.value.as < std::string > () = charsubUTF8(yystack_[3].value.as < std::string > (), pos);
	}
#line 2994 "src/asm/parser.cpp"
    break;

  case 301: // string: "STRCAT" "(" ")"
#line 1578 "src/asm/parser.y"
                                  {
		yylhs.value.as < std::string > ().clear();
	}
#line 3002 "src/asm/parser.cpp"
    break;

  case 302: // string: "STRCAT" "(" strcat_args ")"
#line 1581 "src/asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
	}
#line 3010 "src/asm/parser.cpp"
    break;

  case 303: // string: "STRUPR" "(" string ")"
#line 1584 "src/asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return toupper(c); });
	}
#line 3019 "src/asm/parser.cpp"
    break;

  case 304: // string: "STRLWR" "(" string ")"
#line 1588 "src/asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return tolower(c); });
	}
#line 3028 "src/asm/parser.cpp"
    break;

  case 305: // string: "STRRPL" "(" string "," string "," string ")"
#line 1592 "src/asm/parser.y"
                                                                   {
		yylhs.value.as < std::string > () = strrpl(yystack_[5].value.as < std::string > (), yystack_[3].value.as < std::string > (), yystack_[1].value.as < std::string > ());
	}
#line 3036 "src/asm/parser.cpp"
    break;

  case 306: // string: "STRFMT" "(" strfmt_args ")"
#line 1595 "src/asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = strfmt(yystack_[1].value.as < StrFmtArgList > ().format, yystack_[1].value.as < StrFmtArgList > ().args);
	}
#line 3044 "src/asm/parser.cpp"
    break;

  case 307: // string: "SECTION" "(" scoped_anon_id ")"
#line 1598 "src/asm/parser.y"
                                                   {
		Symbol *sym = sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ());

		if (!sym) {
			if (sym_IsPurgedScoped(yystack_[1].value.as < std::string > ()))
				fatalerror("Unknown symbol \"%s\"; it was purged\n", yystack_[1].value.as < std::string > ().c_str());
			else
				fatalerror("Unknown symbol \"%s\"\n", yystack_[1].value.as < std::string > ().c_str());
		}
		Section const *section = sym->getSection();

		if (!section)
			fatalerror("\"%s\" does not belong to any section\n", sym->name.c_str());
		// Section names are capped by rgbasm's maximum string length,
		// so this currently can't overflow.
		yylhs.value.as < std::string > () = section->name;
	}
#line 3066 "src/asm/parser.cpp"
    break;

  case 308: // strcat_args: string
#line 1618 "src/asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 3074 "src/asm/parser.cpp"
    break;

  case 309: // strcat_args: strcat_args "," string
#line 1621 "src/asm/parser.y"
                                   {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		yylhs.value.as < std::string > ().append(yystack_[0].value.as < std::string > ());
	}
#line 3083 "src/asm/parser.cpp"
    break;

  case 310: // strfmt_args: %empty
#line 1628 "src/asm/parser.y"
                 {}
#line 3089 "src/asm/parser.cpp"
    break;

  case 311: // strfmt_args: string strfmt_va_args
#line 1629 "src/asm/parser.y"
                                {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[0].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().format = std::move(yystack_[1].value.as < std::string > ());
	}
#line 3098 "src/asm/parser.cpp"
    break;

  case 312: // strfmt_va_args: %empty
#line 1636 "src/asm/parser.y"
                 {}
#line 3104 "src/asm/parser.cpp"
    break;

  case 313: // strfmt_va_args: strfmt_va_args "," const_no_str
#line 1637 "src/asm/parser.y"
                                            {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back(static_cast<uint32_t>(yystack_[0].value.as < int32_t > ()));
	}
#line 3113 "src/asm/parser.cpp"
    break;

  case 314: // strfmt_va_args: strfmt_va_args "," string
#line 1641 "src/asm/parser.y"
                                      {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back(std::move(yystack_[0].value.as < std::string > ()));
	}
#line 3122 "src/asm/parser.cpp"
    break;

  case 315: // section: "SECTION" sect_mod string "," sect_type sect_org sect_attrs
#line 1648 "src/asm/parser.y"
                                                                        {
		sect_NewSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 3130 "src/asm/parser.cpp"
    break;

  case 316: // pushs_section: "PUSHS" sect_mod string "," sect_type sect_org sect_attrs
#line 1654 "src/asm/parser.y"
                                                                      {
		sect_PushSection();
		sect_NewSection(yystack_[4].value.as < std::string > (), yystack_[2].value.as < SectionType > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 3139 "src/asm/parser.cpp"
    break;

  case 317: // sect_mod: %empty
#line 1661 "src/asm/parser.y"
               {
		yylhs.value.as < SectionModifier > () = SECTION_NORMAL;
	}
#line 3147 "src/asm/parser.cpp"
    break;

  case 318: // sect_mod: "UNION"
#line 1664 "src/asm/parser.y"
                    {
		yylhs.value.as < SectionModifier > () = SECTION_UNION;
	}
#line 3155 "src/asm/parser.cpp"
    break;

  case 319: // sect_mod: "FRAGMENT"
#line 1667 "src/asm/parser.y"
                       {
		yylhs.value.as < SectionModifier > () = SECTION_FRAGMENT;
	}
#line 3163 "src/asm/parser.cpp"
    break;

  case 320: // sect_type: "WRAM0"
#line 1673 "src/asm/parser.y"
                   {
		yylhs.value.as < SectionType > () = SECTTYPE_WRAM0;
	}
#line 3171 "src/asm/parser.cpp"
    break;

  case 321: // sect_type: "VRAM"
#line 1676 "src/asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_VRAM;
	}
#line 3179 "src/asm/parser.cpp"
    break;

  case 322: // sect_type: "ROMX"
#line 1679 "src/asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_ROMX;
	}
#line 3187 "src/asm/parser.cpp"
    break;

  case 323: // sect_type: "ROM0"
#line 1682 "src/asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_ROM0;
	}
#line 3195 "src/asm/parser.cpp"
    break;

  case 324: // sect_type: "HRAM"
#line 1685 "src/asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_HRAM;
	}
#line 3203 "src/asm/parser.cpp"
    break;

  case 325: // sect_type: "WRAMX"
#line 1688 "src/asm/parser.y"
                     {
		yylhs.value.as < SectionType > () = SECTTYPE_WRAMX;
	}
#line 3211 "src/asm/parser.cpp"
    break;

  case 326: // sect_type: "SRAM"
#line 1691 "src/asm/parser.y"
                    {
		yylhs.value.as < SectionType > () = SECTTYPE_SRAM;
	}
#line 3219 "src/asm/parser.cpp"
    break;

  case 327: // sect_type: "OAM"
#line 1694 "src/asm/parser.y"
                   {
		yylhs.value.as < SectionType > () = SECTTYPE_OAM;
	}
#line 3227 "src/asm/parser.cpp"
    break;

  case 328: // sect_org: %empty
#line 1700 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = -1;
	}
#line 3235 "src/asm/parser.cpp"
    break;

  case 329: // sect_org: "[" uconst "]"
#line 1703 "src/asm/parser.y"
                               {
		yylhs.value.as < int32_t > () = yystack_[1].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0 || yylhs.value.as < int32_t > () > 0xFFFF) {
			::error("Address $%x is not 16-bit\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = -1;
		}
	}
#line 3247 "src/asm/parser.cpp"
    break;

  case 330: // sect_attrs: %empty
#line 1713 "src/asm/parser.y"
               {
		yylhs.value.as < SectionSpec > ().alignment = 0;
		yylhs.value.as < SectionSpec > ().alignOfs = 0;
		yylhs.value.as < SectionSpec > ().bank = -1;
	}
#line 3257 "src/asm/parser.cpp"
    break;

  case 331: // sect_attrs: sect_attrs "," "ALIGN" "[" align_spec "]"
#line 1718 "src/asm/parser.y"
                                                              {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().alignment = yystack_[1].value.as < AlignmentSpec > ().alignment;
		yylhs.value.as < SectionSpec > ().alignOfs = yystack_[1].value.as < AlignmentSpec > ().alignOfs;
	}
#line 3267 "src/asm/parser.cpp"
    break;

  case 332: // sect_attrs: sect_attrs "," "BANK" "[" uconst "]"
#line 1723 "src/asm/parser.y"
                                                        {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().bank = yystack_[1].value.as < int32_t > (); // We cannot check the validity of this yet
	}
#line 3276 "src/asm/parser.cpp"
    break;

  case 381: // z80_adc: "adc" op_a_n
#line 1786 "src/asm/parser.y"
                       {
		sect_ConstByte(0xCE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3285 "src/asm/parser.cpp"
    break;

  case 382: // z80_adc: "adc" op_a_r
#line 1790 "src/asm/parser.y"
                         {
		sect_ConstByte(0x88 | yystack_[0].value.as < int32_t > ());
	}
#line 3293 "src/asm/parser.cpp"
    break;

  case 383: // z80_add: "add" op_a_n
#line 1796 "src/asm/parser.y"
                       {
		sect_ConstByte(0xC6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3302 "src/asm/parser.cpp"
    break;

  case 384: // z80_add: "add" op_a_r
#line 1800 "src/asm/parser.y"
                         {
		sect_ConstByte(0x80 | yystack_[0].value.as < int32_t > ());
	}
#line 3310 "src/asm/parser.cpp"
    break;

  case 385: // z80_add: "add" "hl" "," reg_ss
#line 1803 "src/asm/parser.y"
                                       {
		sect_ConstByte(0x09 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3318 "src/asm/parser.cpp"
    break;

  case 386: // z80_add: "add" "sp" "," reloc_8bit
#line 1806 "src/asm/parser.y"
                                           {
		sect_ConstByte(0xE8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3327 "src/asm/parser.cpp"
    break;

  case 387: // z80_and: "and" op_a_n
#line 1813 "src/asm/parser.y"
                       {
		sect_ConstByte(0xE6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3336 "src/asm/parser.cpp"
    break;

  case 388: // z80_and: "and" op_a_r
#line 1817 "src/asm/parser.y"
                         {
		sect_ConstByte(0xA0 | yystack_[0].value.as < int32_t > ());
	}
#line 3344 "src/asm/parser.cpp"
    break;

  case 389: // z80_bit: "bit" bit_const "," reg_r
#line 1823 "src/asm/parser.y"
                                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3353 "src/asm/parser.cpp"
    break;

  case 390: // z80_call: "call" reloc_16bit
#line 1830 "src/asm/parser.y"
                             {
		sect_ConstByte(0xCD);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3362 "src/asm/parser.cpp"
    break;

  case 391: // z80_call: "call" ccode_expr "," reloc_16bit
#line 1834 "src/asm/parser.y"
                                                {
		sect_ConstByte(0xC4 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3371 "src/asm/parser.cpp"
    break;

  case 392: // z80_ccf: "ccf"
#line 1841 "src/asm/parser.y"
                {
		sect_ConstByte(0x3F);
	}
#line 3379 "src/asm/parser.cpp"
    break;

  case 393: // z80_cp: "cp" op_a_n
#line 1847 "src/asm/parser.y"
                      {
		sect_ConstByte(0xFE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3388 "src/asm/parser.cpp"
    break;

  case 394: // z80_cp: "cp" op_a_r
#line 1851 "src/asm/parser.y"
                        {
		sect_ConstByte(0xB8 | yystack_[0].value.as < int32_t > ());
	}
#line 3396 "src/asm/parser.cpp"
    break;

  case 395: // z80_cpl: "cpl"
#line 1857 "src/asm/parser.y"
                {
		sect_ConstByte(0x2F);
	}
#line 3404 "src/asm/parser.cpp"
    break;

  case 396: // z80_cpl: "cpl" MODE_A
#line 1860 "src/asm/parser.y"
                         {
		sect_ConstByte(0x2F);
	}
#line 3412 "src/asm/parser.cpp"
    break;

  case 397: // z80_daa: "daa"
#line 1866 "src/asm/parser.y"
                {
		sect_ConstByte(0x27);
	}
#line 3420 "src/asm/parser.cpp"
    break;

  case 398: // z80_dec: "dec" reg_r
#line 1872 "src/asm/parser.y"
                      {
		sect_ConstByte(0x05 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3428 "src/asm/parser.cpp"
    break;

  case 399: // z80_dec: "dec" reg_ss
#line 1875 "src/asm/parser.y"
                         {
		sect_ConstByte(0x0B | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3436 "src/asm/parser.cpp"
    break;

  case 400: // z80_di: "di"
#line 1881 "src/asm/parser.y"
               {
		sect_ConstByte(0xF3);
	}
#line 3444 "src/asm/parser.cpp"
    break;

  case 401: // z80_ei: "ei"
#line 1887 "src/asm/parser.y"
               {
		sect_ConstByte(0xFB);
	}
#line 3452 "src/asm/parser.cpp"
    break;

  case 402: // z80_halt: "halt"
#line 1893 "src/asm/parser.y"
                 {
		sect_ConstByte(0x76);
	}
#line 3460 "src/asm/parser.cpp"
    break;

  case 403: // z80_inc: "inc" reg_r
#line 1899 "src/asm/parser.y"
                      {
		sect_ConstByte(0x04 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3468 "src/asm/parser.cpp"
    break;

  case 404: // z80_inc: "inc" reg_ss
#line 1902 "src/asm/parser.y"
                         {
		sect_ConstByte(0x03 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3476 "src/asm/parser.cpp"
    break;

  case 405: // z80_jp: "jp" reloc_16bit
#line 1908 "src/asm/parser.y"
                           {
		sect_ConstByte(0xC3);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3485 "src/asm/parser.cpp"
    break;

  case 406: // z80_jp: "jp" ccode_expr "," reloc_16bit
#line 1912 "src/asm/parser.y"
                                              {
		sect_ConstByte(0xC2 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3494 "src/asm/parser.cpp"
    break;

  case 407: // z80_jp: "jp" "hl"
#line 1916 "src/asm/parser.y"
                         {
		sect_ConstByte(0xE9);
	}
#line 3502 "src/asm/parser.cpp"
    break;

  case 408: // z80_jr: "jr" reloc_16bit
#line 1922 "src/asm/parser.y"
                           {
		sect_ConstByte(0x18);
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3511 "src/asm/parser.cpp"
    break;

  case 409: // z80_jr: "jr" ccode_expr "," reloc_16bit
#line 1926 "src/asm/parser.y"
                                              {
		sect_ConstByte(0x20 | (yystack_[2].value.as < int32_t > () << 3));
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3520 "src/asm/parser.cpp"
    break;

  case 410: // z80_ldi: "ldi" "[" "hl" "]" "," MODE_A
#line 1933 "src/asm/parser.y"
                                                   {
		sect_ConstByte(0x02 | (2 << 4));
	}
#line 3528 "src/asm/parser.cpp"
    break;

  case 411: // z80_ldi: "ldi" MODE_A "," "[" "hl" "]"
#line 1936 "src/asm/parser.y"
                                                     {
		sect_ConstByte(0x0A | (2 << 4));
	}
#line 3536 "src/asm/parser.cpp"
    break;

  case 412: // z80_ldd: "ldd" "[" "hl" "]" "," MODE_A
#line 1942 "src/asm/parser.y"
                                                   {
		sect_ConstByte(0x02 | (3 << 4));
	}
#line 3544 "src/asm/parser.cpp"
    break;

  case 413: // z80_ldd: "ldd" MODE_A "," "[" "hl" "]"
#line 1945 "src/asm/parser.y"
                                                     {
		sect_ConstByte(0x0A | (3 << 4));
	}
#line 3552 "src/asm/parser.cpp"
    break;

  case 414: // z80_ldh: "ldh" MODE_A "," op_mem_ind
#line 1951 "src/asm/parser.y"
                                        {
		if (yystack_[0].value.as < Expression > ().makeCheckHRAM()) {
			warning(
			    WARNING_OBSOLETE,
			    "LDH is deprecated with values from $00 to $FF; use $FF00 to $FFFF\n"
			);
		}

		sect_ConstByte(0xF0);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3568 "src/asm/parser.cpp"
    break;

  case 415: // z80_ldh: "ldh" op_mem_ind "," MODE_A
#line 1962 "src/asm/parser.y"
                                          {
		if (yystack_[2].value.as < Expression > ().makeCheckHRAM()) {
			warning(
			    WARNING_OBSOLETE,
			    "LDH is deprecated with values from $00 to $FF; use $FF00 to $FFFF\n"
			);
		}

		sect_ConstByte(0xE0);
		sect_RelByte(yystack_[2].value.as < Expression > (), 1);
	}
#line 3584 "src/asm/parser.cpp"
    break;

  case 416: // z80_ldh: "ldh" MODE_A "," c_ind
#line 1973 "src/asm/parser.y"
                                     {
		sect_ConstByte(0xF2);
	}
#line 3592 "src/asm/parser.cpp"
    break;

  case 417: // z80_ldh: "ldh" c_ind "," MODE_A
#line 1976 "src/asm/parser.y"
                                     {
		sect_ConstByte(0xE2);
	}
#line 3600 "src/asm/parser.cpp"
    break;

  case 419: // c_ind: "[" relocexpr "+" MODE_C "]"
#line 1983 "src/asm/parser.y"
                                                {
		// This has to use `relocexpr`, not `iconst`, to avoid a shift/reduce conflict
		if (yystack_[3].value.as < Expression > ().getConstVal() != 0xFF00)
			::error("Base value must be equal to $FF00 for $FF00+C\n");
	}
#line 3610 "src/asm/parser.cpp"
    break;

  case 428: // z80_ld_hl: "ld" "hl" "," "sp" reloc_8bit_offset
#line 2002 "src/asm/parser.y"
                                                       {
		sect_ConstByte(0xF8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3619 "src/asm/parser.cpp"
    break;

  case 429: // z80_ld_hl: "ld" "hl" "," reloc_16bit
#line 2006 "src/asm/parser.y"
                                           {
		sect_ConstByte(0x01 | (REG_HL << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3628 "src/asm/parser.cpp"
    break;

  case 430: // z80_ld_sp: "ld" "sp" "," "hl"
#line 2013 "src/asm/parser.y"
                                     {
		sect_ConstByte(0xF9);
	}
#line 3636 "src/asm/parser.cpp"
    break;

  case 431: // z80_ld_sp: "ld" "sp" "," reloc_16bit
#line 2016 "src/asm/parser.y"
                                           {
		sect_ConstByte(0x01 | (REG_SP << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3645 "src/asm/parser.cpp"
    break;

  case 432: // z80_ld_mem: "ld" op_mem_ind "," "sp"
#line 2023 "src/asm/parser.y"
                                        {
		sect_ConstByte(0x08);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3654 "src/asm/parser.cpp"
    break;

  case 433: // z80_ld_mem: "ld" op_mem_ind "," MODE_A
#line 2027 "src/asm/parser.y"
                                         {
		sect_ConstByte(0xEA);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3663 "src/asm/parser.cpp"
    break;

  case 434: // z80_ld_c_ind: "ld" c_ind "," MODE_A
#line 2034 "src/asm/parser.y"
                                  {
		warning(WARNING_OBSOLETE, "LD [C], A is deprecated; use LDH [C], A\n");
		sect_ConstByte(0xE2);
	}
#line 3672 "src/asm/parser.cpp"
    break;

  case 435: // z80_ld_rr: "ld" reg_rr "," MODE_A
#line 2041 "src/asm/parser.y"
                                   {
		sect_ConstByte(0x02 | (yystack_[2].value.as < int32_t > () << 4));
	}
#line 3680 "src/asm/parser.cpp"
    break;

  case 436: // z80_ld_r_no_a: "ld" reg_r_no_a "," reloc_8bit
#line 2047 "src/asm/parser.y"
                                           {
		sect_ConstByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3689 "src/asm/parser.cpp"
    break;

  case 437: // z80_ld_r_no_a: "ld" reg_r_no_a "," reg_r
#line 2051 "src/asm/parser.y"
                                        {
		if (yystack_[2].value.as < int32_t > () == REG_HL_IND && yystack_[0].value.as < int32_t > () == REG_HL_IND)
			::error("LD [HL], [HL] is not a valid instruction\n");
		else
			sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3700 "src/asm/parser.cpp"
    break;

  case 438: // z80_ld_a: "ld" reg_a "," reloc_8bit
#line 2060 "src/asm/parser.y"
                                      {
		sect_ConstByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3709 "src/asm/parser.cpp"
    break;

  case 439: // z80_ld_a: "ld" reg_a "," reg_r
#line 2064 "src/asm/parser.y"
                                   {
		sect_ConstByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3717 "src/asm/parser.cpp"
    break;

  case 440: // z80_ld_a: "ld" reg_a "," c_ind
#line 2067 "src/asm/parser.y"
                                   {
		warning(WARNING_OBSOLETE, "LD A, [C] is deprecated; use LDH A, [C]\n");
		sect_ConstByte(0xF2);
	}
#line 3726 "src/asm/parser.cpp"
    break;

  case 441: // z80_ld_a: "ld" reg_a "," reg_rr
#line 2071 "src/asm/parser.y"
                                    {
		sect_ConstByte(0x0A | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3734 "src/asm/parser.cpp"
    break;

  case 442: // z80_ld_a: "ld" reg_a "," op_mem_ind
#line 2074 "src/asm/parser.y"
                                        {
		sect_ConstByte(0xFA);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3743 "src/asm/parser.cpp"
    break;

  case 443: // z80_ld_ss: "ld" "bc" "," reloc_16bit
#line 2081 "src/asm/parser.y"
                                         {
		sect_ConstByte(0x01 | (REG_BC << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3752 "src/asm/parser.cpp"
    break;

  case 444: // z80_ld_ss: "ld" "de" "," reloc_16bit
#line 2085 "src/asm/parser.y"
                                           {
		sect_ConstByte(0x01 | (REG_DE << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3761 "src/asm/parser.cpp"
    break;

  case 445: // z80_nop: "nop"
#line 2094 "src/asm/parser.y"
                {
		sect_ConstByte(0x00);
	}
#line 3769 "src/asm/parser.cpp"
    break;

  case 446: // z80_or: "or" op_a_n
#line 2100 "src/asm/parser.y"
                      {
		sect_ConstByte(0xF6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3778 "src/asm/parser.cpp"
    break;

  case 447: // z80_or: "or" op_a_r
#line 2104 "src/asm/parser.y"
                        {
		sect_ConstByte(0xB0 | yystack_[0].value.as < int32_t > ());
	}
#line 3786 "src/asm/parser.cpp"
    break;

  case 448: // z80_pop: "pop" reg_tt
#line 2110 "src/asm/parser.y"
                       {
		sect_ConstByte(0xC1 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3794 "src/asm/parser.cpp"
    break;

  case 449: // z80_push: "push" reg_tt
#line 2116 "src/asm/parser.y"
                        {
		sect_ConstByte(0xC5 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3802 "src/asm/parser.cpp"
    break;

  case 450: // z80_res: "res" bit_const "," reg_r
#line 2122 "src/asm/parser.y"
                                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x80 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3811 "src/asm/parser.cpp"
    break;

  case 451: // z80_ret: "ret"
#line 2129 "src/asm/parser.y"
                {
		sect_ConstByte(0xC9);
	}
#line 3819 "src/asm/parser.cpp"
    break;

  case 452: // z80_ret: "ret" ccode_expr
#line 2132 "src/asm/parser.y"
                             {
		sect_ConstByte(0xC0 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3827 "src/asm/parser.cpp"
    break;

  case 453: // z80_reti: "reti"
#line 2138 "src/asm/parser.y"
                 {
		sect_ConstByte(0xD9);
	}
#line 3835 "src/asm/parser.cpp"
    break;

  case 454: // z80_rl: "rl" reg_r
#line 2144 "src/asm/parser.y"
                     {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x10 | yystack_[0].value.as < int32_t > ());
	}
#line 3844 "src/asm/parser.cpp"
    break;

  case 455: // z80_rla: "rla"
#line 2151 "src/asm/parser.y"
                {
		sect_ConstByte(0x17);
	}
#line 3852 "src/asm/parser.cpp"
    break;

  case 456: // z80_rlc: "rlc" reg_r
#line 2157 "src/asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x00 | yystack_[0].value.as < int32_t > ());
	}
#line 3861 "src/asm/parser.cpp"
    break;

  case 457: // z80_rlca: "rlca"
#line 2164 "src/asm/parser.y"
                 {
		sect_ConstByte(0x07);
	}
#line 3869 "src/asm/parser.cpp"
    break;

  case 458: // z80_rr: "rr" reg_r
#line 2170 "src/asm/parser.y"
                     {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x18 | yystack_[0].value.as < int32_t > ());
	}
#line 3878 "src/asm/parser.cpp"
    break;

  case 459: // z80_rra: "rra"
#line 2177 "src/asm/parser.y"
                {
		sect_ConstByte(0x1F);
	}
#line 3886 "src/asm/parser.cpp"
    break;

  case 460: // z80_rrc: "rrc" reg_r
#line 2183 "src/asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x08 | yystack_[0].value.as < int32_t > ());
	}
#line 3895 "src/asm/parser.cpp"
    break;

  case 461: // z80_rrca: "rrca"
#line 2190 "src/asm/parser.y"
                 {
		sect_ConstByte(0x0F);
	}
#line 3903 "src/asm/parser.cpp"
    break;

  case 462: // z80_rst: "rst" reloc_8bit
#line 2196 "src/asm/parser.y"
                           {
		yystack_[0].value.as < Expression > ().makeCheckRST();
		if (!yystack_[0].value.as < Expression > ().isKnown())
			sect_RelByte(yystack_[0].value.as < Expression > (), 0);
		else
			sect_ConstByte(0xC7 | yystack_[0].value.as < Expression > ().value());
	}
#line 3915 "src/asm/parser.cpp"
    break;

  case 463: // z80_sbc: "sbc" op_a_n
#line 2206 "src/asm/parser.y"
                       {
		sect_ConstByte(0xDE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3924 "src/asm/parser.cpp"
    break;

  case 464: // z80_sbc: "sbc" op_a_r
#line 2210 "src/asm/parser.y"
                         {
		sect_ConstByte(0x98 | yystack_[0].value.as < int32_t > ());
	}
#line 3932 "src/asm/parser.cpp"
    break;

  case 465: // z80_scf: "scf"
#line 2216 "src/asm/parser.y"
                {
		sect_ConstByte(0x37);
	}
#line 3940 "src/asm/parser.cpp"
    break;

  case 466: // z80_set: "set" bit_const "," reg_r
#line 2222 "src/asm/parser.y"
                                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0xC0 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3949 "src/asm/parser.cpp"
    break;

  case 467: // z80_sla: "sla" reg_r
#line 2229 "src/asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x20 | yystack_[0].value.as < int32_t > ());
	}
#line 3958 "src/asm/parser.cpp"
    break;

  case 468: // z80_sra: "sra" reg_r
#line 2236 "src/asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x28 | yystack_[0].value.as < int32_t > ());
	}
#line 3967 "src/asm/parser.cpp"
    break;

  case 469: // z80_srl: "srl" reg_r
#line 2243 "src/asm/parser.y"
                      {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x38 | yystack_[0].value.as < int32_t > ());
	}
#line 3976 "src/asm/parser.cpp"
    break;

  case 470: // z80_stop: "stop"
#line 2250 "src/asm/parser.y"
                 {
		sect_ConstByte(0x10);
		sect_ConstByte(0x00);
	}
#line 3985 "src/asm/parser.cpp"
    break;

  case 471: // z80_stop: "stop" reloc_8bit
#line 2254 "src/asm/parser.y"
                              {
		sect_ConstByte(0x10);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3994 "src/asm/parser.cpp"
    break;

  case 472: // z80_sub: "sub" op_a_n
#line 2261 "src/asm/parser.y"
                       {
		sect_ConstByte(0xD6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4003 "src/asm/parser.cpp"
    break;

  case 473: // z80_sub: "sub" op_a_r
#line 2265 "src/asm/parser.y"
                         {
		sect_ConstByte(0x90 | yystack_[0].value.as < int32_t > ());
	}
#line 4011 "src/asm/parser.cpp"
    break;

  case 474: // z80_swap: "swap" reg_r
#line 2271 "src/asm/parser.y"
                       {
		sect_ConstByte(0xCB);
		sect_ConstByte(0x30 | yystack_[0].value.as < int32_t > ());
	}
#line 4020 "src/asm/parser.cpp"
    break;

  case 475: // z80_xor: "xor" op_a_n
#line 2278 "src/asm/parser.y"
                       {
		sect_ConstByte(0xEE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 4029 "src/asm/parser.cpp"
    break;

  case 476: // z80_xor: "xor" op_a_r
#line 2282 "src/asm/parser.y"
                         {
		sect_ConstByte(0xA8 | yystack_[0].value.as < int32_t > ());
	}
#line 4037 "src/asm/parser.cpp"
    break;

  case 477: // op_mem_ind: "[" reloc_16bit "]"
#line 2290 "src/asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 4045 "src/asm/parser.cpp"
    break;

  case 478: // op_a_r: reg_r
#line 2296 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4051 "src/asm/parser.cpp"
    break;

  case 479: // op_a_r: MODE_A "," reg_r
#line 2297 "src/asm/parser.y"
                             {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
	}
#line 4059 "src/asm/parser.cpp"
    break;

  case 480: // op_a_n: reloc_8bit
#line 2303 "src/asm/parser.y"
                   {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 4067 "src/asm/parser.cpp"
    break;

  case 481: // op_a_n: MODE_A "," reloc_8bit
#line 2306 "src/asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 4075 "src/asm/parser.cpp"
    break;

  case 496: // ccode_expr: ccode
#line 2349 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4081 "src/asm/parser.cpp"
    break;

  case 497: // ccode_expr: "!" ccode_expr
#line 2350 "src/asm/parser.y"
                                 {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > () ^ 1;
	}
#line 4089 "src/asm/parser.cpp"
    break;

  case 498: // ccode: "nz"
#line 2356 "src/asm/parser.y"
              {
		yylhs.value.as < int32_t > () = CC_NZ;
	}
#line 4097 "src/asm/parser.cpp"
    break;

  case 499: // ccode: "z"
#line 2359 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = CC_Z;
	}
#line 4105 "src/asm/parser.cpp"
    break;

  case 500: // ccode: "nc"
#line 2362 "src/asm/parser.y"
                {
		yylhs.value.as < int32_t > () = CC_NC;
	}
#line 4113 "src/asm/parser.cpp"
    break;

  case 501: // ccode: "c"
#line 2365 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = CC_C;
	}
#line 4121 "src/asm/parser.cpp"
    break;

  case 502: // reg_r: reg_r_no_a
#line 2370 "src/asm/parser.y"
       { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4127 "src/asm/parser.cpp"
    break;

  case 503: // reg_r: reg_a
#line 2370 "src/asm/parser.y"
                    { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 4133 "src/asm/parser.cpp"
    break;

  case 504: // reg_r_no_a: MODE_B
#line 2373 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_B;
	}
#line 4141 "src/asm/parser.cpp"
    break;

  case 505: // reg_r_no_a: MODE_C
#line 2376 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_C;
	}
#line 4149 "src/asm/parser.cpp"
    break;

  case 506: // reg_r_no_a: MODE_D
#line 2379 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_D;
	}
#line 4157 "src/asm/parser.cpp"
    break;

  case 507: // reg_r_no_a: MODE_E
#line 2382 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_E;
	}
#line 4165 "src/asm/parser.cpp"
    break;

  case 508: // reg_r_no_a: MODE_H
#line 2385 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_H;
	}
#line 4173 "src/asm/parser.cpp"
    break;

  case 509: // reg_r_no_a: MODE_L
#line 2388 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_L;
	}
#line 4181 "src/asm/parser.cpp"
    break;

  case 510: // reg_r_no_a: "[" "hl" "]"
#line 2391 "src/asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_HL_IND;
	}
#line 4189 "src/asm/parser.cpp"
    break;

  case 511: // reg_a: MODE_A
#line 2397 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_A;
	}
#line 4197 "src/asm/parser.cpp"
    break;

  case 512: // reg_tt: "bc"
#line 2403 "src/asm/parser.y"
                {
		yylhs.value.as < int32_t > () = REG_BC;
	}
#line 4205 "src/asm/parser.cpp"
    break;

  case 513: // reg_tt: "de"
#line 2406 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_DE;
	}
#line 4213 "src/asm/parser.cpp"
    break;

  case 514: // reg_tt: "hl"
#line 2409 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_HL;
	}
#line 4221 "src/asm/parser.cpp"
    break;

  case 515: // reg_tt: "af"
#line 2412 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_AF;
	}
#line 4229 "src/asm/parser.cpp"
    break;

  case 516: // reg_ss: "bc"
#line 2418 "src/asm/parser.y"
                {
		yylhs.value.as < int32_t > () = REG_BC;
	}
#line 4237 "src/asm/parser.cpp"
    break;

  case 517: // reg_ss: "de"
#line 2421 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_DE;
	}
#line 4245 "src/asm/parser.cpp"
    break;

  case 518: // reg_ss: "hl"
#line 2424 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_HL;
	}
#line 4253 "src/asm/parser.cpp"
    break;

  case 519: // reg_ss: "sp"
#line 2427 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_SP;
	}
#line 4261 "src/asm/parser.cpp"
    break;

  case 520: // reg_rr: "[" "bc" "]"
#line 2433 "src/asm/parser.y"
                              {
		yylhs.value.as < int32_t > () = REG_BC_IND;
	}
#line 4269 "src/asm/parser.cpp"
    break;

  case 521: // reg_rr: "[" "de" "]"
#line 2436 "src/asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_DE_IND;
	}
#line 4277 "src/asm/parser.cpp"
    break;

  case 522: // reg_rr: hl_ind_inc
#line 2439 "src/asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDINC;
	}
#line 4285 "src/asm/parser.cpp"
    break;

  case 523: // reg_rr: hl_ind_dec
#line 2442 "src/asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDDEC;
	}
#line 4293 "src/asm/parser.cpp"
    break;


#line 4297 "src/asm/parser.cpp"

            default:
              break;
            }
        }
#if YY_EXCEPTIONS
      catch (const syntax_error& yyexc)
        {
          YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
          error (yyexc);
          YYERROR;
        }
#endif // YY_EXCEPTIONS
      YY_SYMBOL_PRINT ("-> $$ =", yylhs);
      yypop_ (yylen);
      yylen = 0;

      // Shift the result of the reduction.
      yypush_ (YY_NULLPTR, YY_MOVE (yylhs));
    }
    goto yynewstate;


  /*--------------------------------------.
  | yyerrlab -- here on detecting error.  |
  `--------------------------------------*/
  yyerrlab:
    // If not already recovering from an error, report this error.
    if (!yyerrstatus_)
      {
        ++yynerrs_;
        context yyctx (*this, yyla);
        std::string msg = yysyntax_error_ (yyctx);
        error (YY_MOVE (msg));
      }


    if (yyerrstatus_ == 3)
      {
        /* If just tried and failed to reuse lookahead token after an
           error, discard it.  */

        // Return failure if at end of input.
        if (yyla.kind () == symbol_kind::S_YYEOF)
          YYABORT;
        else if (!yyla.empty ())
          {
            yy_destroy_ ("Error: discarding", yyla);
            yyla.clear ();
          }
      }

    // Else will try to reuse lookahead token after shifting the error token.
    goto yyerrlab1;


  /*---------------------------------------------------.
  | yyerrorlab -- error raised explicitly by YYERROR.  |
  `---------------------------------------------------*/
  yyerrorlab:
    /* Pacify compilers when the user code never invokes YYERROR and
       the label yyerrorlab therefore never appears in user code.  */
    if (false)
      YYERROR;

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYERROR.  */
    yypop_ (yylen);
    yylen = 0;
    YY_STACK_PRINT ();
    goto yyerrlab1;


  /*-------------------------------------------------------------.
  | yyerrlab1 -- common code for both syntax error and YYERROR.  |
  `-------------------------------------------------------------*/
  yyerrlab1:
    yyerrstatus_ = 3;   // Each real token shifted decrements this.
    // Pop stack until we find a state that shifts the error token.
    for (;;)
      {
        yyn = yypact_[+yystack_[0].state];
        if (!yy_pact_value_is_default_ (yyn))
          {
            yyn += symbol_kind::S_YYerror;
            if (0 <= yyn && yyn <= yylast_
                && yycheck_[yyn] == symbol_kind::S_YYerror)
              {
                yyn = yytable_[yyn];
                if (0 < yyn)
                  break;
              }
          }

        // Pop the current state because it cannot handle the error token.
        if (yystack_.size () == 1)
          YYABORT;

        yy_destroy_ ("Error: popping", yystack_[0]);
        yypop_ ();
        YY_STACK_PRINT ();
      }
    {
      stack_symbol_type error_token;


      // Shift the error token.
      yy_lac_discard_ ("error recovery");
      error_token.state = state_type (yyn);
      yypush_ ("Shifting", YY_MOVE (error_token));
    }
    goto yynewstate;


  /*-------------------------------------.
  | yyacceptlab -- YYACCEPT comes here.  |
  `-------------------------------------*/
  yyacceptlab:
    yyresult = 0;
    goto yyreturn;


  /*-----------------------------------.
  | yyabortlab -- YYABORT comes here.  |
  `-----------------------------------*/
  yyabortlab:
    yyresult = 1;
    goto yyreturn;


  /*-----------------------------------------------------.
  | yyreturn -- parsing is finished, return the result.  |
  `-----------------------------------------------------*/
  yyreturn:
    if (!yyla.empty ())
      yy_destroy_ ("Cleanup: discarding lookahead", yyla);

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYABORT or YYACCEPT.  */
    yypop_ (yylen);
    YY_STACK_PRINT ();
    while (1 < yystack_.size ())
      {
        yy_destroy_ ("Cleanup: popping", yystack_[0]);
        yypop_ ();
      }

    return yyresult;
  }
#if YY_EXCEPTIONS
    catch (...)
      {
        YYCDEBUG << "Exception caught: cleaning lookahead and stack\n";
        // Do not try to display the values of the reclaimed symbols,
        // as their printers might throw an exception.
        if (!yyla.empty ())
          yy_destroy_ (YY_NULLPTR, yyla);

        while (1 < yystack_.size ())
          {
            yy_destroy_ (YY_NULLPTR, yystack_[0]);
            yypop_ ();
          }
        throw;
      }
#endif // YY_EXCEPTIONS
  }

  void
  parser::error (const syntax_error& yyexc)
  {
    error (yyexc.what ());
  }

  const char *
  parser::symbol_name (symbol_kind_type yysymbol)
  {
    static const char *const yy_sname[] =
    {
    "end of file", "error", "invalid token", "end of line", "end of buffer",
  ",", ":", "::", "[", "]", "(", ")", "+", "-", "*", "/", "%", "**", "==",
  "!=", "<", ">", "<=", ">=", "&&", "||", "!", "&", "|", "^", "<<", ">>",
  ">>>", "~", "NEG", "=", "+=", "-=", "*=", "/=", "%=", "&=", "|=", "^=",
  "<<=", ">>=", "a", "b", "c", "d", "e", "h", "l", "af", "bc", "de", "hl",
  "sp", "hli/hl+", "hld/hl-", "z", "nz", "nc", "adc", "add", "and", "bit",
  "call", "ccf", "cp", "cpl", "daa", "dec", "di", "ei", "halt", "inc",
  "jp", "jr", "ldd", "ldh", "ldi", "ld", "nop", "or", "pop", "push", "res",
  "reti", "ret", "rla", "rlca", "rlc", "rl", "rra", "rrca", "rrc", "rr",
  "rst", "sbc", "scf", "set", "sla", "sra", "srl", "stop", "sub", "swap",
  "xor", "ALIGN", "ASSERT", "BREAK", "CHARMAP", "DB", "DL", "DS", "DW",
  "ELIF", "ELSE", "ENDC", "ENDL", "ENDM", "ENDR", "ENDSECTION", "ENDU",
  "EQU", "EQUS", "EXPORT", "FAIL", "FATAL", "FOR", "FRAGMENT", "IF",
  "INCBIN", "INCLUDE", "LOAD", "MACRO", "NEWCHARMAP", "NEXTU", "OPT",
  "POPC", "POPO", "POPS", "PRINTLN", "PRINT", "PURGE", "PUSHC", "PUSHO",
  "PUSHS", "RB", "REDEF", "REPT", "RSRESET", "RSSET", "RW", "SECTION",
  "SETCHARMAP", "SHIFT", "STATIC_ASSERT", "UNION", "WARN", "ACOS", "ASIN",
  "ATAN", "ATAN2", "BANK", "BITWIDTH", "CEIL", "CHARLEN", "CHARSUB", "COS",
  "DEF", "FDIV", "FLOOR", "FMOD", "FMUL", "HIGH", "INCHARMAP", "ISCONST",
  "LOG", "LOW", "POW", "ROUND", "SIN", "SIZEOF", "STARTOF", "STRCAT",
  "STRCMP", "STRFMT", "STRIN", "STRLEN", "STRLWR", "STRRIN", "STRRPL",
  "STRSUB", "STRUPR", "TAN", "TZCOUNT", "HRAM", "OAM", "ROM0", "ROMX",
  "SRAM", "VRAM", "WRAM0", "WRAMX", "number", "string", "label",
  "identifier", "local identifier", "anonymous label", "$accept",
  "asm_file", "lines", "$@1", "diff_mark", "line", "endofline",
  "line_directive", "if", "elif", "else", "plain_directive", "endc",
  "def_id", "$@2", "redef_id", "$@3", "scoped_id", "scoped_anon_id",
  "label", "macro", "$@4", "macro_args", "directive", "def_numeric",
  "trailing_comma", "compound_eq", "align", "align_spec", "opt", "$@5",
  "opt_list", "opt_list_entry", "popo", "pusho", "$@6", "pusho_opt_list",
  "pops", "pushs", "endsection", "fail", "warn", "assert_type", "assert",
  "shift", "shift_const", "load", "rept", "for", "$@7", "$@8",
  "capture_rept", "for_args", "break", "def_macro", "$@9", "$@10",
  "capture_macro", "rsset", "rsreset", "rs_uconst", "union", "nextu",
  "endu", "ds", "ds_args", "db", "dw", "dl", "def_equ", "redef_equ",
  "def_set", "def_rb", "def_rw", "def_rl", "def_equs", "redef_equs",
  "purge", "$@11", "purge_args", "export", "export_list",
  "export_list_entry", "export_def", "include", "incbin", "charmap",
  "charmap_args", "newcharmap", "setcharmap", "pushc", "pushc_setcharmap",
  "popc", "print", "println", "print_exprs", "print_expr", "bit_const",
  "constlist_8bit", "constlist_8bit_entry", "constlist_16bit",
  "constlist_16bit_entry", "constlist_32bit", "constlist_32bit_entry",
  "reloc_8bit", "reloc_8bit_no_str", "reloc_8bit_offset", "reloc_16bit",
  "reloc_16bit_no_str", "relocexpr", "relocexpr_no_str", "$@12", "uconst",
  "iconst", "const_no_str", "precision_arg", "string", "strcat_args",
  "strfmt_args", "strfmt_va_args", "section", "pushs_section", "sect_mod",
  "sect_type", "sect_org", "sect_attrs", "cpu_commands", "cpu_command",
  "z80_adc", "z80_add", "z80_and", "z80_bit", "z80_call", "z80_ccf",
  "z80_cp", "z80_cpl", "z80_daa", "z80_dec", "z80_di", "z80_ei",
  "z80_halt", "z80_inc", "z80_jp", "z80_jr", "z80_ldi", "z80_ldd",
  "z80_ldh", "c_ind", "z80_ld", "z80_ld_hl", "z80_ld_sp", "z80_ld_mem",
  "z80_ld_c_ind", "z80_ld_rr", "z80_ld_r_no_a", "z80_ld_a", "z80_ld_ss",
  "z80_nop", "z80_or", "z80_pop", "z80_push", "z80_res", "z80_ret",
  "z80_reti", "z80_rl", "z80_rla", "z80_rlc", "z80_rlca", "z80_rr",
  "z80_rra", "z80_rrc", "z80_rrca", "z80_rst", "z80_sbc", "z80_scf",
  "z80_set", "z80_sla", "z80_sra", "z80_srl", "z80_stop", "z80_sub",
  "z80_swap", "z80_xor", "op_mem_ind", "op_a_r", "op_a_n", "MODE_A",
  "MODE_B", "MODE_C", "MODE_D", "MODE_E", "MODE_H", "MODE_L", "ccode_expr",
  "ccode", "reg_r", "reg_r_no_a", "reg_a", "reg_tt", "reg_ss", "reg_rr",
  "hl_ind_inc", "hl_ind_dec", YY_NULLPTR
    };
    return yy_sname[yysymbol];
  }



  // parser::context.
  parser::context::context (const parser& yyparser, const symbol_type& yyla)
    : yyparser_ (yyparser)
    , yyla_ (yyla)
  {}

  int
  parser::context::expected_tokens (symbol_kind_type yyarg[], int yyargn) const
  {
    // Actual number of expected tokens
    int yycount = 0;

#if YYDEBUG
    // Execute LAC once. We don't care if it is successful, we
    // only do it for the sake of debugging output.
    if (!yyparser_.yy_lac_established_)
      yyparser_.yy_lac_check_ (yyla_.kind ());
#endif

    for (int yyx = 0; yyx < YYNTOKENS; ++yyx)
      {
        symbol_kind_type yysym = YY_CAST (symbol_kind_type, yyx);
        if (yysym != symbol_kind::S_YYerror
            && yysym != symbol_kind::S_YYUNDEF
            && yyparser_.yy_lac_check_ (yysym))
          {
            if (!yyarg)
              ++yycount;
            else if (yycount == yyargn)
              return 0;
            else
              yyarg[yycount++] = yysym;
          }
      }
    if (yyarg && yycount == 0 && 0 < yyargn)
      yyarg[0] = symbol_kind::S_YYEMPTY;
    return yycount;
  }




  bool
  parser::yy_lac_check_ (symbol_kind_type yytoken) const
  {
    // Logically, the yylac_stack's lifetime is confined to this function.
    // Clear it, to get rid of potential left-overs from previous call.
    yylac_stack_.clear ();
    // Reduce until we encounter a shift and thereby accept the token.
#if YYDEBUG
    YYCDEBUG << "LAC: checking lookahead " << symbol_name (yytoken) << ':';
#endif
    std::ptrdiff_t lac_top = 0;
    while (true)
      {
        state_type top_state = (yylac_stack_.empty ()
                                ? yystack_[lac_top].state
                                : yylac_stack_.back ());
        int yyrule = yypact_[+top_state];
        if (yy_pact_value_is_default_ (yyrule)
            || (yyrule += yytoken) < 0 || yylast_ < yyrule
            || yycheck_[yyrule] != yytoken)
          {
            // Use the default action.
            yyrule = yydefact_[+top_state];
            if (yyrule == 0)
              {
                YYCDEBUG << " Err\n";
                return false;
              }
          }
        else
          {
            // Use the action from yytable.
            yyrule = yytable_[yyrule];
            if (yy_table_value_is_error_ (yyrule))
              {
                YYCDEBUG << " Err\n";
                return false;
              }
            if (0 < yyrule)
              {
                YYCDEBUG << " S" << yyrule << '\n';
                return true;
              }
            yyrule = -yyrule;
          }
        // By now we know we have to simulate a reduce.
        YYCDEBUG << " R" << yyrule - 1;
        // Pop the corresponding number of values from the stack.
        {
          std::ptrdiff_t yylen = yyr2_[yyrule];
          // First pop from the LAC stack as many tokens as possible.
          std::ptrdiff_t lac_size = std::ptrdiff_t (yylac_stack_.size ());
          if (yylen < lac_size)
            {
              yylac_stack_.resize (std::size_t (lac_size - yylen));
              yylen = 0;
            }
          else if (lac_size)
            {
              yylac_stack_.clear ();
              yylen -= lac_size;
            }
          // Only afterwards look at the main stack.
          // We simulate popping elements by incrementing lac_top.
          lac_top += yylen;
        }
        // Keep top_state in sync with the updated stack.
        top_state = (yylac_stack_.empty ()
                     ? yystack_[lac_top].state
                     : yylac_stack_.back ());
        // Push the resulting state of the reduction.
        state_type state = yy_lr_goto_state_ (top_state, yyr1_[yyrule]);
        YYCDEBUG << " G" << int (state);
        yylac_stack_.push_back (state);
      }
  }

  // Establish the initial context if no initial context currently exists.
  bool
  parser::yy_lac_establish_ (symbol_kind_type yytoken)
  {
    /* Establish the initial context for the current lookahead if no initial
       context is currently established.

       We define a context as a snapshot of the parser stacks.  We define
       the initial context for a lookahead as the context in which the
       parser initially examines that lookahead in order to select a
       syntactic action.  Thus, if the lookahead eventually proves
       syntactically unacceptable (possibly in a later context reached via a
       series of reductions), the initial context can be used to determine
       the exact set of tokens that would be syntactically acceptable in the
       lookahead's place.  Moreover, it is the context after which any
       further semantic actions would be erroneous because they would be
       determined by a syntactically unacceptable token.

       yy_lac_establish_ should be invoked when a reduction is about to be
       performed in an inconsistent state (which, for the purposes of LAC,
       includes consistent states that don't know they're consistent because
       their default reductions have been disabled).

       For parse.lac=full, the implementation of yy_lac_establish_ is as
       follows.  If no initial context is currently established for the
       current lookahead, then check if that lookahead can eventually be
       shifted if syntactic actions continue from the current context.  */
    if (yy_lac_established_)
      return true;
    else
      {
#if YYDEBUG
        YYCDEBUG << "LAC: initial context established for "
                 << symbol_name (yytoken) << '\n';
#endif
        yy_lac_established_ = true;
        return yy_lac_check_ (yytoken);
      }
  }

  // Discard any previous initial lookahead context.
  void
  parser::yy_lac_discard_ (const char* event)
  {
   /* Discard any previous initial lookahead context because of Event,
      which may be a lookahead change or an invalidation of the currently
      established initial context for the current lookahead.

      The most common example of a lookahead change is a shift.  An example
      of both cases is syntax error recovery.  That is, a syntax error
      occurs when the lookahead is syntactically erroneous for the
      currently established initial context, so error recovery manipulates
      the parser stacks to try to find a new initial context in which the
      current lookahead is syntactically acceptable.  If it fails to find
      such a context, it discards the lookahead.  */
    if (yy_lac_established_)
      {
        YYCDEBUG << "LAC: initial context discarded due to "
                 << event << '\n';
        yy_lac_established_ = false;
      }
  }


  int
  parser::yy_syntax_error_arguments_ (const context& yyctx,
                                                 symbol_kind_type yyarg[], int yyargn) const
  {
    /* There are many possibilities here to consider:
       - If this state is a consistent state with a default action, then
         the only way this function was invoked is if the default action
         is an error action.  In that case, don't check for expected
         tokens because there are none.
       - The only way there can be no lookahead present (in yyla) is
         if this state is a consistent state with a default action.
         Thus, detecting the absence of a lookahead is sufficient to
         determine that there is no unexpected or expected token to
         report.  In that case, just report a simple "syntax error".
       - Don't assume there isn't a lookahead just because this state is
         a consistent state with a default action.  There might have
         been a previous inconsistent state, consistent state with a
         non-default action, or user semantic action that manipulated
         yyla.  (However, yyla is currently not documented for users.)
         In the first two cases, it might appear that the current syntax
         error should have been detected in the previous state when
         yy_lac_check was invoked.  However, at that time, there might
         have been a different syntax error that discarded a different
         initial context during error recovery, leaving behind the
         current lookahead.
    */

    if (!yyctx.lookahead ().empty ())
      {
        if (yyarg)
          yyarg[0] = yyctx.token ();
        int yyn = yyctx.expected_tokens (yyarg ? yyarg + 1 : yyarg, yyargn - 1);
        return yyn + 1;
      }
    return 0;
  }

  // Generate an error message.
  std::string
  parser::yysyntax_error_ (const context& yyctx) const
  {
    // Its maximum.
    enum { YYARGS_MAX = 5 };
    // Arguments of yyformat.
    symbol_kind_type yyarg[YYARGS_MAX];
    int yycount = yy_syntax_error_arguments_ (yyctx, yyarg, YYARGS_MAX);

    char const* yyformat = YY_NULLPTR;
    switch (yycount)
      {
#define YYCASE_(N, S)                         \
        case N:                               \
          yyformat = S;                       \
        break
      default: // Avoid compiler warnings.
        YYCASE_ (0, YY_("syntax error"));
        YYCASE_ (1, YY_("syntax error, unexpected %s"));
        YYCASE_ (2, YY_("syntax error, unexpected %s, expecting %s"));
        YYCASE_ (3, YY_("syntax error, unexpected %s, expecting %s or %s"));
        YYCASE_ (4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
        YYCASE_ (5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
#undef YYCASE_
      }

    std::string yyres;
    // Argument number.
    std::ptrdiff_t yyi = 0;
    for (char const* yyp = yyformat; *yyp; ++yyp)
      if (yyp[0] == '%' && yyp[1] == 's' && yyi < yycount)
        {
          yyres += symbol_name (yyarg[yyi++]);
          ++yyp;
        }
      else
        yyres += *yyp;
    return yyres;
  }


  const short parser::yypact_ninf_ = -661;

  const short parser::yytable_ninf_ = -315;

  const short
  parser::yypact_[] =
  {
     903,  -661,    31,    75,   118,  -661,  -661,  -661,    42,  -661,
    -661,  -661,  -661,  2587,    65,  -661,  2587,  -661,  2587,   205,
     264,  -661,  -661,  -661,  -661,  -661,   118,  2736,  -661,  -661,
    -661,  -661,  -661,  2587,  2587,  2587,  2587,  2587,    68,    79,
      82,    84,    94,   107,   109,   114,   119,   128,   133,  -661,
     137,   146,   156,   165,   169,   171,   173,   176,   182,   192,
     194,   204,   217,   241,   247,   255,   281,   284,   304,   320,
     321,   348,   349,   355,   357,   358,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  1572,  -661,   370,  -661,  -661,   166,
     386,   184,   388,  -661,  -661,  -661,  -661,  -661,  -661,  1115,
    1058,  1115,  2587,  1607,  -661,  1115,   -23,  -661,   186,  -661,
    -661,  -661,   186,  1550,  1607,    12,    34,    37,   212,  -661,
    1115,   168,   168,  2587,  -661,    36,  -661,  -661,   149,   149,
    -661,  -661,   149,   149,  2587,  1115,  -661,  2587,   149,   149,
     149,  2587,  1115,   149,  1115,  2587,   -79,   118,   421,  2587,
    2587,  2040,  2587,  -661,  -661,  -661,  -661,   -78,   421,   421,
     421,   -74,   185,  -661,  -661,  -661,  -661,  -661,  2587,  2587,
    -661,   188,  -661,    49,  -661,  -661,  2587,   -74,   190,  2587,
     -79,  -661,   421,  -661,  -661,  -661,   452,   238,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,   296,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
     829,   389,   389,   389,   389,    45,  2587,  2587,  2587,  2587,
     455,  2587,  2587,   421,   421,  2587,   397,  2587,  2587,  2587,
    2587,  2587,   421,  2587,  2587,  2587,  2587,  2587,  2587,   571,
     571,    -4,   421,   421,   421,   421,   421,   421,   421,   421,
     421,  2587,  2587,  2587,  2587,  2587,  2587,  2587,  2587,  2587,
    2587,  2587,  2587,  2587,  2587,  2587,  2587,  2587,  2587,  2587,
    2587,  2587,  2587,  -661,  -661,  -661,  -661,  -661,   366,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,   413,   415,  -661,  1572,
    -661,  -661,   450,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,   454,   457,  -661,  -661,  -661,  -661,   466,  -661,
    1607,  -661,  -661,  -661,  -661,  -661,  1572,   468,  -661,  -661,
    -661,   416,  -661,  -661,  -661,  -661,  -661,   464,   467,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,   473,  -661,   474,   424,
     476,  2097,   478,   480,   494,   444,   496,  1362,   498,   500,
     501,   502,   503,   506,   507,   508,   515,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,   516,    36,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,   519,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,   520,   523,
     534,   535,  2587,  -661,   538,   541,  -661,  -661,  1572,   159,
     244,   543,  -661,   313,   323,   544,   549,   550,  -661,  -661,
     367,   439,   393,   373,  -661,  -661,   557,  -661,  -661,   561,
     118,  -661,  -661,   421,   562,   342,   563,  -661,   105,  -661,
     453,   563,   257,  -661,   342,   421,   360,  -661,   421,  -661,
    -661,  -661,  2587,  -661,   364,  -661,  2587,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  2587,  2587,   421,
    2587,  2587,  2587,  2587,  2587,   421,  2587,  2558,  -661,   568,
     575,   575,   575,   580,   576,   577,   925,   575,   578,   581,
     575,    45,   588,   575,   592,   593,  1157,   589,  1179,   598,
    1411,   599,   575,   575,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,   594,   597,   600,   602,  -661,  -661,   134,   612,
    -661,   607,   614,   609,   610,   617,   618,   625,   620,   575,
    1659,   721,   721,   389,   389,   389,   389,   666,   666,   666,
     666,   666,   666,  2062,  1897,   214,   214,   214,   293,   293,
     293,   628,   631,   118,   626,  1795,  1852,  1115,   267,  2587,
     149,  -661,  2587,   585,   300,   414,  2587,  2587,   630,   632,
     634,   633,  1629,   636,   -23,   -23,   639,   642,   644,   650,
     658,   164,   659,   661,  2587,  2587,  2285,  2342,   -23,     9,
    1115,  1305,   -23,   149,   149,  2587,  -661,  -661,  -661,   799,
    2587,  2587,  -661,  2587,  -661,  2587,  2587,  2587,  -661,   257,
    -661,  2587,  -661,   667,   465,  -661,   342,  -661,  2587,  -661,
    -661,  -661,   670,   342,  -661,   679,  -661,   680,   681,  -661,
     482,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  2587,   676,   688,   689,  2587,
    -661,  -661,  -661,   690,  -661,  2587,   691,   693,  2587,   694,
    2587,  2587,  -661,  -661,  -661,  2587,  -661,  2587,   695,   697,
    -661,  -661,  -661,  -661,   421,  -661,   421,   685,  -661,   421,
    -661,  -661,   421,   421,  2587,  -661,   698,  -661,  2587,  -661,
    -661,  -661,   699,   701,   703,   704,   706,   709,   710,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,   686,   668,  2530,
    -661,  2097,  -661,  -661,  -661,  -661,  -661,   717,   671,  -661,
    -661,   720,   722,  -661,  -661,  -661,  -661,   158,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,   421,   727,  -661,  -661,  -661,
     724,   734,  -661,  -661,  -661,   736,   248,  -661,  -661,  -661,
     257,  -661,   248,   248,   421,  -661,  -661,  -661,  -661,  -661,
     575,  -661,   732,  -661,  -661,   575,  -661,   575,   575,   575,
     575,  -661,  -661,  -661,   733,  2587,   743,   744,   742,   135,
    -661,   753,   756,   118,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,   -23,   749,   754,   -23,   758,  -661,  -661,  2587,  2587,
    -661,  -661,  2587,  -661,   763,  2587,  -661,  2587,   769,  -661,
     769,   769,  -661,   771,  -661,   773,   774,   776,   777,   778,
    -661,  -661,   150,  -661,  -661,   421,  2587,  -661,  -661,  2587,
    -661,  -661,  -661,  -661,  -661,  -661,  1572,  1572,  -661,  2587,
    -661,  -661,  -661,  2587,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,   779,   780,   118,   775,   734,   783,   788,
     788,   788,  -661,  -661,  -661,  2587,  -661,  -661,   -83,  -661,
     787,   789,  2587,  2587,   790,   791,  -661,  -661
  };

  const short
  parser::yydefact_[] =
  {
       0,     5,     0,     7,     0,     1,     8,     9,    39,    12,
      13,     6,    40,     0,     0,   139,     0,   147,     0,     0,
      41,     4,    11,    19,    20,    21,     0,    25,    15,    16,
      17,    14,    18,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   268,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   232,   297,    36,    34,
      35,    38,    37,   231,   293,   229,     0,   230,    24,     0,
       0,     0,     0,   292,    43,    45,    42,    44,    10,     0,
       0,     0,     0,     0,   392,     0,   395,   397,     0,   400,
     401,   402,     0,     0,     0,     0,     0,     0,     0,   445,
       0,     0,     0,     0,   453,   451,   455,   457,     0,     0,
     459,   461,     0,     0,     0,     0,   465,     0,     0,     0,
       0,   470,     0,     0,     0,     0,   125,     0,     0,   164,
     168,     0,   166,    29,   137,   122,   157,     0,     0,     0,
       0,   317,     0,   156,   110,   202,   115,   120,   204,     0,
     181,   200,   116,   317,    32,   152,     0,   317,     0,   134,
     125,   155,     0,    30,    46,    50,     0,     0,    27,    28,
      77,    88,    87,    85,    86,    81,    82,    84,    74,    75,
      76,    73,    72,    61,    60,    62,    63,    64,    58,    55,
      56,    57,    89,    90,    91,    92,    93,    94,    78,    79,
      80,    53,    54,    65,    66,    67,    68,    69,    71,    70,
      51,    52,    59,    83,    26,   333,   335,   336,   337,   338,
     339,   340,   341,   342,   343,   344,   345,   346,   347,   348,
     349,   350,   354,   352,   353,   351,   424,   425,   420,   421,
     422,   426,   427,   423,   355,   356,   357,   358,   359,   360,
     361,   362,   363,   364,   365,   366,   367,   368,   369,   370,
     371,   372,   373,   374,   375,   376,   377,   378,   379,   380,
       0,   254,   255,   233,   256,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   310,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    23,   140,    22,   148,   142,     0,   482,
     484,   486,   488,   490,   492,   494,     0,     0,   480,   223,
     382,   381,   511,   504,   505,   506,   507,   508,   509,   478,
     502,   503,     0,     0,   384,   383,   388,   387,     0,   210,
       0,   501,   499,   498,   500,   390,   227,     0,   496,   394,
     393,     0,   396,   516,   517,   518,   519,     0,     0,   511,
     398,   399,   403,   404,   407,   405,     0,   408,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   522,   523,   447,
     446,   515,   512,   513,   514,   448,   449,     0,     0,   452,
     456,   454,   460,   458,   462,   464,   463,     0,   467,   468,
     469,   471,   473,   472,   474,   476,   475,   107,   108,     0,
       0,     0,     0,   146,     0,    95,   211,   213,     0,   229,
     230,    95,   219,   229,   230,     0,   158,    95,   215,   217,
     229,   230,     0,     0,   188,   189,    95,   186,   123,   191,
       0,   319,   318,     0,   197,     0,    95,   206,   229,   208,
     230,    95,     0,   201,   118,     0,     0,   151,     0,   199,
     133,   135,     0,   124,     0,    48,     0,    97,    98,    99,
     100,   101,   104,   103,   102,   105,   106,   153,     0,     0,
     153,   153,     0,     0,     0,     0,     0,     0,   291,     0,
     295,   295,   295,     0,     0,     0,     0,   295,     0,     0,
     295,     0,     0,   295,     0,     0,     0,     0,     0,     0,
       0,     0,   295,   295,   324,   327,   323,   322,   326,   321,
     320,   325,     0,     0,     0,     0,   301,   308,     0,     0,
     312,     0,     0,     0,     0,     0,     0,     0,     0,   295,
       0,   242,   243,   250,   251,   252,   253,   236,   241,   238,
     237,   240,   239,   235,   234,   246,   245,   244,   247,   248,
     249,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   497,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   227,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   127,   128,   126,   129,
       0,    96,   165,    96,   169,     0,     0,    96,   167,    96,
     185,     0,   190,     0,     0,   114,   111,   112,    96,   205,
     203,   183,    95,   119,   117,     0,    33,     0,   131,    31,
      47,   172,   178,   154,   170,   179,   176,   177,   174,   173,
     171,   180,   175,   334,   307,     0,     0,     0,     0,     0,
     262,   263,   259,     0,   289,     0,     0,     0,     0,     0,
       0,     0,   257,   290,   261,     0,   258,     0,     0,     0,
     264,   266,   265,   267,     0,   302,     0,   311,   306,     0,
     288,   304,     0,     0,     0,   303,     0,   260,     0,   150,
     138,   510,     0,     0,     0,     0,     0,     0,     0,   481,
     479,   385,   386,   389,   391,   406,   409,     0,     0,     0,
     477,     0,   418,   417,   415,   416,   414,     0,     0,   520,
     521,     0,     0,   524,   526,   443,   444,     0,   429,   430,
     431,   434,   432,   433,   436,   437,   438,   440,   442,   439,
     441,   435,   450,   466,   109,     0,    95,   195,   212,   220,
       0,    95,   162,   216,   187,   192,     0,   198,   113,   207,
      96,   182,     0,     0,     0,    49,   296,   282,   281,   283,
     295,   271,     0,   279,   269,   295,   272,   295,   295,   295,
     295,   270,   278,   309,     0,     0,     0,     0,     0,     0,
     280,     0,   143,     0,   483,   485,   489,   493,   487,   491,
     495,     0,     0,     0,     0,     0,   525,   527,     0,     0,
     428,   130,    96,   194,    95,    96,   159,     0,   328,   184,
     328,   328,   132,     0,   300,     0,     0,     0,     0,     0,
     285,   313,   230,   286,   287,     0,     0,   299,   142,     0,
     149,   412,   413,   419,   410,   411,   225,   226,   196,    96,
     160,   163,   193,     0,   330,   330,   330,   284,   273,   275,
     274,   277,   276,     0,     0,     0,   144,    95,     0,   136,
     316,   315,   305,   298,   141,     0,   161,   329,     0,   145,
       0,     0,     0,     0,     0,     0,   331,   332
  };

  const short
  parser::yypgoto_[] =
  {
    -661,  -661,  -661,  -661,  -661,  -661,   -24,  -661,  -661,  -661,
    -661,  -661,  -661,   641,  -661,   645,  -661,  -138,  -249,  -661,
    -661,  -661,  -661,  -661,   648,  -420,  -173,  -661,  -660,  -661,
    -661,   282,  -591,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,   627,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,   -76,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -322,  -661,  -661,  -661,  -661,  -111,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,   154,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,   656,   147,   -58,  -661,   163,
    -661,   160,  -661,   189,  -125,  -661,  -661,   -73,  -661,     0,
    -144,  -661,   -18,   -12,   -19,  -445,    57,  -661,  -661,  -661,
    -661,  -661,  -100,  -317,  -602,  -626,   291,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -108,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,  -661,
    -661,  -661,  -661,   -96,   319,   340,   -88,  -661,  -383,  -661,
    -661,  -661,  -661,    23,  -661,   -69,   745,   746,   747,   -91,
     206,  -661,  -661
  };

  const short
  parser::yydefgoto_[] =
  {
       0,     2,     3,     4,     8,    21,    11,    22,    23,    24,
      25,    26,   185,   186,   524,   187,   516,    82,    83,    27,
     188,   525,   700,   189,   190,   672,   542,   191,   467,   192,
     505,   686,   687,   193,   194,   514,   694,   195,   196,   197,
     198,   199,   472,   200,   201,   520,   202,    28,    29,    89,
     621,   623,   861,    30,    31,    91,   622,   863,   203,   204,
     702,   205,   206,   207,   208,   821,   209,   210,   211,   212,
     213,   214,   215,   216,   217,   218,   219,   220,   512,   692,
     221,   496,   497,   222,    32,   223,   224,   816,   225,   226,
     227,   228,   229,   230,   231,   506,   507,   388,   475,   476,
     487,   488,   481,   482,   368,   477,   880,   641,   489,    84,
      85,   306,   468,    93,   509,   716,    87,   588,   591,   747,
     232,   233,   503,   583,   924,   939,   234,   235,   236,   237,
     238,   239,   240,   241,   242,   243,   244,   245,   246,   247,
     248,   249,   250,   251,   252,   253,   254,   422,   255,   256,
     257,   258,   259,   260,   261,   262,   263,   264,   265,   266,
     267,   268,   269,   270,   271,   272,   273,   274,   275,   276,
     277,   278,   279,   280,   281,   282,   283,   284,   285,   286,
     287,   288,   289,   423,   370,   371,   409,   373,   374,   375,
     376,   377,   378,   631,   398,   379,   380,   381,   445,   411,
     436,   437,   438
  };

  const short
  parser::yytable_[] =
  {
      92,    86,    98,   585,    90,   479,   483,   586,   490,   454,
     432,   372,   372,   372,   546,   820,   461,   372,   402,   494,
     419,   413,   433,   359,   508,   508,   950,   420,   424,   426,
     395,     5,   372,   290,   291,   292,   293,   294,   643,   410,
     415,   417,   421,   412,   643,   425,   549,   372,    12,   469,
     470,   554,  -121,  -121,   372,   359,   372,   501,   359,   450,
     451,   674,   448,   452,   453,   447,   802,   678,    88,   458,
     459,   460,   174,   515,   464,    -2,   680,   518,   295,   457,
     359,   471,   951,   359,   391,   502,   689,     6,     7,   296,
     389,   690,   297,   183,   298,   828,   392,   393,   394,   369,
     369,   369,   828,   396,   299,   369,   717,   718,  -294,  -294,
    -294,   389,   723,   396,   396,   726,  -294,   300,   729,   301,
     369,     9,    10,   473,   302,   389,   397,   738,   739,   303,
      78,    79,    80,   486,   369,   369,   416,   418,   304,   744,
     906,   369,   369,   305,   369,   745,   907,   307,   449,   478,
     478,    38,   478,   401,   756,  -314,   308,   358,   517,    13,
      14,  -314,  -224,  -224,  -224,    47,   309,   521,   478,   478,
     878,   879,    15,   761,    16,   310,   791,   792,    17,   311,
     501,   312,    64,   313,    66,   401,   314,    69,   401,    71,
      72,    73,   315,    18,   358,   359,   360,   361,   362,   363,
     364,   365,   316,    77,   317,   474,   480,   484,   502,   491,
     401,    94,    95,   401,   318,   498,   499,   500,   706,   707,
     427,   441,   442,   443,   444,   510,   510,   319,   335,   336,
     337,   338,   359,   360,   361,   362,   363,   364,   365,   523,
     403,   404,   405,   406,   350,   351,   352,  -214,  -214,  -214,
      19,   320,    20,    78,    79,    80,    81,   321,   359,   360,
     361,   362,   363,   364,   365,   322,   428,   429,   430,   431,
      96,    97,   831,   543,   527,   528,   529,   530,   531,   532,
     533,   534,   535,   536,   550,   551,   552,   553,   925,   926,
     557,   323,   954,   560,   324,   562,   563,   564,   565,   940,
     941,   556,   569,   547,   571,   572,   573,   335,   336,   337,
     338,   566,   727,   568,   325,   570,  -221,  -221,  -221,   599,
     546,   403,   404,   405,   406,   407,  -222,  -222,  -222,   408,
     326,   327,   600,   601,   602,   603,   604,   605,   606,   607,
     608,   609,   610,   611,   612,   613,   614,   615,   616,   617,
     618,   619,   620,   762,   763,   764,   765,   555,   328,   329,
     558,   559,   407,   544,   545,   330,   408,   331,   332,   567,
    -228,  -228,  -228,   353,   691,   354,   582,   584,   587,   589,
     590,   592,   593,   594,   595,   596,   597,   598,   407,   355,
     293,   357,   408,   356,   504,   893,   883,   513,   873,   519,
     895,   886,   896,   897,   898,   899,   338,   561,   543,   527,
     528,   529,   530,   531,   532,   533,   534,   535,   536,   384,
     386,   642,   624,   625,   399,   626,   633,   642,   526,   527,
     528,   529,   530,   531,   532,   533,   534,   535,   536,   439,
     385,   387,  -218,  -218,  -218,   400,   574,   575,   576,   577,
     578,   579,   580,   581,   455,   627,  -209,  -209,  -209,   628,
     440,   462,   629,   465,   920,    78,    79,    80,   766,   767,
     768,   630,   669,   632,   634,   456,   682,   635,   636,   637,
     638,   639,   463,   644,   466,   645,   537,   526,   527,   528,
     529,   530,   531,   532,   533,   534,   535,   536,   544,   646,
     647,   648,   769,   654,   772,   655,   656,   657,   658,   888,
     698,   659,   660,   661,   701,   890,   891,   946,   538,   703,
     662,   663,   703,   703,   664,   665,   704,   479,   666,   483,
     708,   709,   710,   490,   712,   804,   806,   771,   785,   667,
     668,   494,   540,   670,   508,   537,   671,   541,   673,   685,
     786,   822,   675,   807,   676,   677,   783,   784,   770,   774,
     683,   773,   679,   775,   776,   808,   681,   684,   688,   696,
     801,   803,   695,   699,   811,   697,    38,   538,   539,   714,
     715,   795,   796,   798,   800,   719,   725,   720,   721,   724,
      47,   805,   809,   728,   812,   813,   705,   730,   731,   760,
     733,   540,   711,   735,   737,   740,   541,    64,   741,    66,
      38,   742,    69,   743,    71,    72,    73,   746,   748,   749,
     750,   751,   752,   753,    47,   566,   570,   369,    77,   369,
     754,   755,   396,   758,   759,   761,   396,   396,   762,   777,
     778,    64,   780,    66,   779,   782,    69,   421,    71,    72,
      73,   787,   788,   814,   396,   396,   396,   396,   817,   789,
     369,   369,    77,    78,    79,    80,    81,   790,   793,   825,
     794,   478,   826,   478,   827,   830,   369,   478,   333,   334,
     335,   336,   337,   338,   832,   833,   834,   837,   478,   835,
     855,   871,   889,   347,   348,   349,   350,   351,   352,   838,
     839,   841,   843,   836,   844,   846,   851,   840,   852,   860,
     864,   508,   865,   842,   866,   867,   845,   868,   847,   848,
     869,   870,   874,   849,   872,   850,    38,   875,   480,   876,
     484,   877,   882,   884,   491,   335,   336,   337,   338,   885,
      47,   887,   859,   894,   900,   510,   862,   905,   347,   348,
     349,   350,   351,   352,   903,   904,   908,    64,   912,    66,
     921,   909,    69,   913,    71,    72,    73,   915,   919,   574,
     575,   576,   577,   578,   579,   580,   581,   923,    77,   570,
     945,   601,   927,   911,   928,   929,   914,   930,   931,   932,
     942,   943,   947,   948,   822,   952,   693,   953,   492,   956,
     957,   853,   493,   854,   815,   495,   856,   522,   937,   857,
     858,   333,   334,   335,   336,   337,   338,   339,   340,   341,
     342,   343,   344,   345,   346,   511,   347,   348,   349,   350,
     351,   352,   935,   824,   818,   829,   901,   823,   713,   910,
     548,   333,   334,   335,   336,   337,   338,   339,   340,   341,
     342,   343,   344,   345,   346,   478,   347,   348,   349,   350,
     351,   352,   819,   434,   435,     0,     0,   810,     0,   446,
     918,     0,   881,     0,     0,   922,     0,     0,   916,   917,
       0,     0,     0,     0,     0,   369,     0,     0,   934,     0,
       0,   892,     0,     0,     0,     0,     0,   936,     0,     0,
       0,     0,     0,    -3,     1,   938,    -3,    -3,     0,    -3,
       0,   944,   902,     0,     0,    -3,    -3,     0,     0,   369,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   949,     0,   955,   722,   333,   334,   335,
     336,   337,   338,   339,   340,   341,   342,   343,   344,   345,
     346,     0,   347,   348,   349,   350,   351,   352,     0,     0,
       0,     0,   933,     0,     0,     0,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,     0,     0,    -3,    -3,     0,     0,
      -3,    -3,     0,    -3,     0,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
      -3,    -3,     0,    -3,    -3,    -3,    -3,     0,    -3,    -3,
      -3,    -3,    -3,    -3,     0,     0,   358,     0,    33,     0,
      34,    35,     0,     0,    -3,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    36,     0,     0,     0,     0,     0,
       0,    37,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   359,   360,   361,   362,   363,   364,
     365,    -3,    -3,    -3,   382,   383,     0,     0,     0,     0,
       0,     0,     0,   358,     0,    33,     0,    34,    35,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,    36,     0,     0,     0,     0,     0,     0,    37,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   359,   360,   361,   362,   363,   364,   365,   732,   333,
     334,   335,   336,   337,   338,   339,   340,   341,   342,   343,
     344,   345,   346,     0,   347,   348,   349,   350,   351,   352,
     734,   333,   334,   335,   336,   337,   338,   339,   340,   341,
     342,   343,   344,   345,   346,     0,   347,   348,   349,   350,
     351,   352,     0,    38,     0,     0,     0,     0,     0,    39,
      40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
      50,    51,    52,    53,   366,    55,    56,    57,   367,    59,
      60,    61,    62,    63,    64,    65,    66,    67,    68,    69,
      70,    71,    72,    73,    74,    75,     0,     0,     0,     0,
       0,     0,     0,     0,    76,    77,    78,    79,    80,    81,
      38,     0,     0,     0,     0,     0,    39,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,   366,    55,    56,    57,   367,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,   427,     0,    33,     0,    34,    35,     0,
       0,    76,    77,    78,    79,    80,    81,     0,     0,     0,
       0,    36,     0,     0,     0,     0,     0,     0,    37,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   359,   360,   361,   362,   363,   364,   365,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    33,     0,    34,    35,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    36,     0,
       0,     0,     0,     0,     0,    37,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     361,     0,     0,     0,     0,     0,   649,   650,   651,     0,
     652,   653,   736,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,   346,     0,   347,   348,
     349,   350,   351,   352,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      38,     0,     0,     0,     0,     0,    39,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,   366,    55,    56,    57,   367,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,     0,     0,     0,     0,     0,     0,     0,
       0,    76,    77,    78,    79,    80,    81,    38,     0,     0,
       0,     0,     0,    39,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,   640,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      33,     0,    34,    35,     0,     0,     0,     0,    76,    77,
      78,    79,    80,    81,     0,     0,   390,     0,     0,     0,
       0,     0,     0,    37,   333,   334,   335,   336,   337,   338,
     339,   340,   341,   342,   343,   344,   345,   346,   391,   347,
     348,   349,   350,   351,   352,     0,   414,     0,     0,     0,
     392,   393,   394,     0,     0,     0,     0,    33,     0,    34,
      35,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   390,     0,     0,     0,     0,     0,     0,
      37,   781,   334,   335,   336,   337,   338,   339,   340,   341,
     342,   343,   344,   345,   346,   391,   347,   348,   349,   350,
     351,   352,     0,     0,     0,     0,     0,   392,   393,   394,
     757,   333,   334,   335,   336,   337,   338,   339,   340,   341,
     342,   343,   344,   345,   346,     0,   347,   348,   349,   350,
     351,   352,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    38,     0,     0,     0,     0,
       0,    39,    40,    41,    42,    43,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,    54,    55,    56,    57,
      58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,     0,     0,
       0,     0,     0,     0,     0,     0,    76,    77,    78,    79,
      80,    81,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,    33,     0,    34,    35,     0,
       0,     0,     0,    76,    77,    78,    79,    80,    81,     0,
       0,    36,     0,     0,     0,     0,     0,     0,    37,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   762,   763,
     764,   765,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    33,     0,    34,    35,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    36,     0,
       0,     0,     0,     0,     0,    37,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   766,   767,   768,   333,
     334,   335,   336,   337,   338,   339,   340,   341,   342,   343,
     344,   345,     0,     0,   347,   348,   349,   350,   351,   352,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      38,     0,     0,     0,     0,     0,    39,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,     0,     0,     0,     0,     0,     0,     0,
       0,    76,    77,    78,    79,    80,    81,    38,     0,     0,
       0,     0,     0,    39,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      33,     0,    34,    35,     0,     0,     0,     0,    76,    77,
      78,    79,    80,    81,     0,     0,    36,     0,     0,     0,
       0,     0,     0,    37,   333,   334,   335,   336,   337,   338,
     339,   340,   341,   342,   343,   344,     0,     0,     0,   347,
     348,   349,   350,   351,   352,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    33,     0,    34,
      35,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    36,     0,     0,     0,     0,     0,     0,
      37,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   361,     0,     0,     0,   485,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    38,     0,     0,     0,     0,
       0,    39,    40,    41,    42,    43,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,    54,    55,    56,    57,
      58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,     0,     0,
       0,     0,     0,     0,     0,     0,    76,    77,    78,    79,
      80,    81,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,   640,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,    33,     0,    34,    35,     0,
       0,     0,     0,    76,    77,    78,    79,    80,    81,     0,
       0,    36,     0,     0,     0,     0,     0,     0,    37,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   797,     0,     0,     0,     0,     0,     0,     0,
       0,     0,    33,     0,    34,    35,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    36,     0,
       0,     0,     0,     0,     0,    37,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   799,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      38,     0,     0,     0,     0,     0,    39,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,     0,     0,     0,     0,     0,     0,     0,
       0,    76,    77,    78,    79,    80,    81,    38,     0,     0,
       0,     0,     0,    39,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      33,     0,    34,    35,     0,     0,     0,     0,    76,    77,
      78,    79,    80,    81,     0,     0,    36,     0,     0,     0,
       0,     0,     0,    37,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   766,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    33,     0,    34,
      35,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    36,     0,     0,     0,     0,     0,     0,
      37,    99,   100,   101,   102,   103,   104,   105,   106,   107,
     108,   109,   110,   111,   112,   113,   114,   115,   116,   117,
     118,   119,   120,   121,   122,   123,   124,   125,   126,   127,
     128,   129,   130,   131,   132,   133,   134,   135,   136,   137,
     138,   139,   140,   141,   142,   143,   144,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,    38,     0,     0,     0,     0,
       0,    39,    40,    41,    42,    43,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,    54,    55,    56,    57,
      58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,     0,     0,
       0,     0,     0,     0,     0,     0,    76,    77,    78,    79,
      80,    81,    38,     0,     0,     0,     0,     0,    39,    40,
      41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,     0,     0,     0,     0,     0,
       0,     0,     0,    76,    77,    78,    79,    80,    81,    99,
     100,   101,   102,   103,   104,   105,   106,   107,   108,   109,
     110,   111,   112,   113,   114,   115,   116,   117,   118,   119,
     120,   121,   122,   123,   124,   125,   126,   127,   128,   129,
     130,   131,   132,   133,   134,   135,   136,   137,   138,   139,
     140,   141,   142,   143,   144,   145,   146,   147,   148,   149,
     150,   151,   152,     0,     0,   153,   154,     0,     0,   155,
     156,     0,     0,   157,   158,     0,     0,     0,     0,   159,
     160,   161,     0,   162,   163,   164,   165,   166,   167,   168,
     169,   170,   171,   172,   173,     0,   174,     0,   175,   176,
       0,   177,   178,   179,   180,   181,   182,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   183,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   184
  };

  const short
  parser::yycheck_[] =
  {
      18,    13,    26,   320,    16,   149,   150,    11,   152,   134,
     118,    99,   100,   101,   187,   675,   141,   105,   106,   157,
       8,   112,   118,    46,   168,   169,   109,   115,   116,   117,
     103,     0,   120,    33,    34,    35,    36,    37,   421,   108,
     113,   114,     8,   112,   427,     8,   295,   135,     6,   128,
     129,   300,     3,     4,   142,    46,   144,   131,    46,   128,
     129,   481,    26,   132,   133,   123,    57,   487,     3,   138,
     139,   140,   150,   173,   143,     0,   496,   177,    10,   137,
      46,   160,   165,    46,    48,   159,   506,    12,    13,    10,
     102,   511,    10,   171,    10,   686,    60,    61,    62,    99,
     100,   101,   693,   103,    10,   105,   551,   552,     3,     4,
       5,   123,   557,   113,   114,   560,    11,    10,   563,    10,
     120,     3,     4,   147,    10,   137,   103,   572,   573,    10,
     208,   209,   210,   151,   134,   135,   113,   114,    10,     5,
       5,   141,   142,    10,   144,    11,    11,    10,   125,   149,
     150,   155,   152,   176,   599,     5,    10,     8,   176,   117,
     118,    11,     3,     4,     5,   169,    10,   179,   168,   169,
      12,    13,   130,     9,   132,    10,    12,    13,   136,    10,
     131,    10,   186,    10,   188,   176,    10,   191,   176,   193,
     194,   195,    10,   151,     8,    46,    47,    48,    49,    50,
      51,    52,    10,   207,    10,   148,   149,   150,   159,   152,
     176,     6,     7,   176,    10,   158,   159,   160,   540,   541,
       8,    53,    54,    55,    56,   168,   169,    10,    14,    15,
      16,    17,    46,    47,    48,    49,    50,    51,    52,   182,
      54,    55,    56,    57,    30,    31,    32,     3,     4,     5,
     208,    10,   210,   208,   209,   210,   211,    10,    46,    47,
      48,    49,    50,    51,    52,    10,    54,    55,    56,    57,
       6,     7,   692,    35,    36,    37,    38,    39,    40,    41,
      42,    43,    44,    45,   296,   297,   298,   299,   890,   891,
     302,    10,   952,   305,    10,   307,   308,   309,   310,   925,
     926,   301,   314,     7,   316,   317,   318,    14,    15,    16,
      17,   311,   561,   313,    10,   315,     3,     4,     5,   331,
     493,    54,    55,    56,    57,   176,     3,     4,     5,   180,
      10,    10,   332,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,   346,   347,   348,   349,
     350,   351,   352,    53,    54,    55,    56,   300,    10,    10,
     303,   304,   176,   125,   126,    10,   180,    10,    10,   312,
       3,     4,     5,     3,   512,   209,   319,   320,   321,   322,
     323,   324,   325,   326,   327,   328,   329,   330,   176,     3,
     390,     3,   180,   209,   209,   840,   816,   209,   781,   209,
     845,   821,   847,   848,   849,   850,    17,    10,    35,    36,
      37,    38,    39,    40,    41,    42,    43,    44,    45,   100,
     101,   421,    56,    10,   105,    10,    10,   427,    35,    36,
      37,    38,    39,    40,    41,    42,    43,    44,    45,   120,
     100,   101,     3,     4,     5,   105,   198,   199,   200,   201,
     202,   203,   204,   205,   135,     5,     3,     4,     5,     5,
     120,   142,     5,   144,   884,   208,   209,   210,    54,    55,
      56,     5,   472,     5,    10,   135,   500,    10,     5,     5,
      56,     5,   142,     5,   144,     5,    93,    35,    36,    37,
      38,    39,    40,    41,    42,    43,    44,    45,   125,     5,
      56,     5,   627,     5,   629,     5,     5,     5,     5,   826,
     522,     5,     5,     5,   526,   832,   833,   937,   125,   537,
       5,     5,   540,   541,     5,     5,   538,   671,     5,   673,
     542,   543,   544,   677,   546,   660,   661,   628,   646,     5,
       5,   679,   149,     5,   688,    93,     5,   154,     5,   207,
     646,   676,     8,   661,     5,     5,   644,   645,   627,   632,
     503,   630,     5,   636,   637,   661,     5,     5,     5,   209,
     658,   659,   515,   209,   662,   518,   155,   125,   126,    11,
       5,   654,   655,   656,   657,     5,     5,    11,    11,    11,
     169,   660,   661,     5,   663,   664,   539,     5,     5,   623,
      11,   149,   545,     5,     5,    11,   154,   186,    11,   188,
     155,    11,   191,    11,   193,   194,   195,     5,    11,     5,
      11,    11,     5,     5,   169,   625,   626,   627,   207,   629,
       5,    11,   632,     5,     3,     9,   636,   637,    53,     9,
       8,   186,     9,   188,    10,     9,   191,     8,   193,   194,
     195,     9,     8,   665,   654,   655,   656,   657,   670,     9,
     660,   661,   207,   208,   209,   210,   211,     9,     9,   681,
       9,   671,     5,   673,   209,     5,   676,   677,    12,    13,
      14,    15,    16,    17,     5,     5,     5,    11,   688,   207,
       5,     5,   830,    27,    28,    29,    30,    31,    32,    11,
      11,    11,    11,   715,    11,    11,    11,   719,    11,    11,
      11,   855,    11,   725,    11,    11,   728,    11,   730,   731,
      11,    11,     5,   735,    56,   737,   155,    56,   671,     9,
     673,     9,     5,     9,   677,    14,    15,    16,    17,     5,
     169,     5,   754,    11,    11,   688,   758,     5,    27,    28,
      29,    30,    31,    32,    11,    11,     3,   186,     9,   188,
     885,     5,   191,     9,   193,   194,   195,     9,     5,   198,
     199,   200,   201,   202,   203,   204,   205,     8,   207,   779,
       5,   781,    11,   871,    11,    11,   874,    11,    11,    11,
      11,    11,     9,     5,   919,     8,   514,     8,   157,     9,
       9,   744,   157,   746,     5,   157,   749,   180,   919,   752,
     753,    12,    13,    14,    15,    16,    17,    18,    19,    20,
      21,    22,    23,    24,    25,   169,    27,    28,    29,    30,
      31,    32,   908,   679,   671,   688,   855,   677,   547,   863,
      11,    12,    13,    14,    15,    16,    17,    18,    19,    20,
      21,    22,    23,    24,    25,   855,    27,    28,    29,    30,
      31,    32,   673,   118,   118,    -1,    -1,   661,    -1,   122,
     882,    -1,   815,    -1,    -1,   887,    -1,    -1,   878,   879,
      -1,    -1,    -1,    -1,    -1,   885,    -1,    -1,   906,    -1,
      -1,   834,    -1,    -1,    -1,    -1,    -1,   909,    -1,    -1,
      -1,    -1,    -1,     0,     1,   923,     3,     4,    -1,     6,
      -1,   935,   855,    -1,    -1,    12,    13,    -1,    -1,   919,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   945,    -1,   953,    11,    12,    13,    14,
      15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
      25,    -1,    27,    28,    29,    30,    31,    32,    -1,    -1,
      -1,    -1,   905,    -1,    -1,    -1,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,    81,    82,    83,    84,    85,    86,
      87,    88,    89,    90,    91,    92,    93,    94,    95,    96,
      97,    98,    99,   100,   101,   102,   103,   104,   105,   106,
     107,   108,   109,   110,   111,   112,   113,   114,   115,   116,
     117,   118,   119,   120,    -1,    -1,   123,   124,    -1,    -1,
     127,   128,    -1,   130,    -1,   132,   133,   134,   135,   136,
     137,   138,   139,   140,   141,   142,   143,   144,   145,   146,
     147,   148,    -1,   150,   151,   152,   153,    -1,   155,   156,
     157,   158,   159,   160,    -1,    -1,     8,    -1,    10,    -1,
      12,    13,    -1,    -1,   171,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,
      -1,    33,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    46,    47,    48,    49,    50,    51,
      52,   208,   209,   210,    56,    57,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,     8,    -1,    10,    -1,    12,    13,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,    33,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    46,    47,    48,    49,    50,    51,    52,    11,    12,
      13,    14,    15,    16,    17,    18,    19,    20,    21,    22,
      23,    24,    25,    -1,    27,    28,    29,    30,    31,    32,
      11,    12,    13,    14,    15,    16,    17,    18,    19,    20,
      21,    22,    23,    24,    25,    -1,    27,    28,    29,    30,
      31,    32,    -1,   155,    -1,    -1,    -1,    -1,    -1,   161,
     162,   163,   164,   165,   166,   167,   168,   169,   170,   171,
     172,   173,   174,   175,   176,   177,   178,   179,   180,   181,
     182,   183,   184,   185,   186,   187,   188,   189,   190,   191,
     192,   193,   194,   195,   196,   197,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   206,   207,   208,   209,   210,   211,
     155,    -1,    -1,    -1,    -1,    -1,   161,   162,   163,   164,
     165,   166,   167,   168,   169,   170,   171,   172,   173,   174,
     175,   176,   177,   178,   179,   180,   181,   182,   183,   184,
     185,   186,   187,   188,   189,   190,   191,   192,   193,   194,
     195,   196,   197,     8,    -1,    10,    -1,    12,    13,    -1,
      -1,   206,   207,   208,   209,   210,   211,    -1,    -1,    -1,
      -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,    33,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    46,    47,    48,    49,    50,    51,    52,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    26,    -1,
      -1,    -1,    -1,    -1,    -1,    33,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      48,    -1,    -1,    -1,    -1,    -1,    54,    55,    56,    -1,
      58,    59,    11,    12,    13,    14,    15,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    25,    -1,    27,    28,
      29,    30,    31,    32,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     155,    -1,    -1,    -1,    -1,    -1,   161,   162,   163,   164,
     165,   166,   167,   168,   169,   170,   171,   172,   173,   174,
     175,   176,   177,   178,   179,   180,   181,   182,   183,   184,
     185,   186,   187,   188,   189,   190,   191,   192,   193,   194,
     195,   196,   197,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   206,   207,   208,   209,   210,   211,   155,    -1,    -1,
      -1,    -1,    -1,   161,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
      10,    -1,    12,    13,    -1,    -1,    -1,    -1,   206,   207,
     208,   209,   210,   211,    -1,    -1,    26,    -1,    -1,    -1,
      -1,    -1,    -1,    33,    12,    13,    14,    15,    16,    17,
      18,    19,    20,    21,    22,    23,    24,    25,    48,    27,
      28,    29,    30,    31,    32,    -1,    56,    -1,    -1,    -1,
      60,    61,    62,    -1,    -1,    -1,    -1,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      33,    12,    13,    14,    15,    16,    17,    18,    19,    20,
      21,    22,    23,    24,    25,    48,    27,    28,    29,    30,
      31,    32,    -1,    -1,    -1,    -1,    -1,    60,    61,    62,
      11,    12,    13,    14,    15,    16,    17,    18,    19,    20,
      21,    22,    23,    24,    25,    -1,    27,    28,    29,    30,
      31,    32,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   155,    -1,    -1,    -1,    -1,
      -1,   161,   162,   163,   164,   165,   166,   167,   168,   169,
     170,   171,   172,   173,   174,   175,   176,   177,   178,   179,
     180,   181,   182,   183,   184,   185,   186,   187,   188,   189,
     190,   191,   192,   193,   194,   195,   196,   197,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   206,   207,   208,   209,
     210,   211,   155,    -1,    -1,    -1,    -1,    -1,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,    10,    -1,    12,    13,    -1,
      -1,    -1,    -1,   206,   207,   208,   209,   210,   211,    -1,
      -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,    33,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    53,    54,
      55,    56,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    26,    -1,
      -1,    -1,    -1,    -1,    -1,    33,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    54,    55,    56,    12,
      13,    14,    15,    16,    17,    18,    19,    20,    21,    22,
      23,    24,    -1,    -1,    27,    28,    29,    30,    31,    32,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     155,    -1,    -1,    -1,    -1,    -1,   161,   162,   163,   164,
     165,   166,   167,   168,   169,   170,   171,   172,   173,   174,
     175,   176,   177,   178,   179,   180,   181,   182,   183,   184,
     185,   186,   187,   188,   189,   190,   191,   192,   193,   194,
     195,   196,   197,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   206,   207,   208,   209,   210,   211,   155,    -1,    -1,
      -1,    -1,    -1,   161,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
      10,    -1,    12,    13,    -1,    -1,    -1,    -1,   206,   207,
     208,   209,   210,   211,    -1,    -1,    26,    -1,    -1,    -1,
      -1,    -1,    -1,    33,    12,    13,    14,    15,    16,    17,
      18,    19,    20,    21,    22,    23,    -1,    -1,    -1,    27,
      28,    29,    30,    31,    32,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      33,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    48,    -1,    -1,    -1,   109,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   155,    -1,    -1,    -1,    -1,
      -1,   161,   162,   163,   164,   165,   166,   167,   168,   169,
     170,   171,   172,   173,   174,   175,   176,   177,   178,   179,
     180,   181,   182,   183,   184,   185,   186,   187,   188,   189,
     190,   191,   192,   193,   194,   195,   196,   197,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   206,   207,   208,   209,
     210,   211,   155,    -1,    -1,    -1,    -1,    -1,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,    10,    -1,    12,    13,    -1,
      -1,    -1,    -1,   206,   207,   208,   209,   210,   211,    -1,
      -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,    33,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    57,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    10,    -1,    12,    13,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    26,    -1,
      -1,    -1,    -1,    -1,    -1,    33,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    56,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     155,    -1,    -1,    -1,    -1,    -1,   161,   162,   163,   164,
     165,   166,   167,   168,   169,   170,   171,   172,   173,   174,
     175,   176,   177,   178,   179,   180,   181,   182,   183,   184,
     185,   186,   187,   188,   189,   190,   191,   192,   193,   194,
     195,   196,   197,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   206,   207,   208,   209,   210,   211,   155,    -1,    -1,
      -1,    -1,    -1,   161,   162,   163,   164,   165,   166,   167,
     168,   169,   170,   171,   172,   173,   174,   175,   176,   177,
     178,   179,   180,   181,   182,   183,   184,   185,   186,   187,
     188,   189,   190,   191,   192,   193,   194,   195,   196,   197,
      10,    -1,    12,    13,    -1,    -1,    -1,    -1,   206,   207,
     208,   209,   210,   211,    -1,    -1,    26,    -1,    -1,    -1,
      -1,    -1,    -1,    33,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    54,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    10,    -1,    12,
      13,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    26,    -1,    -1,    -1,    -1,    -1,    -1,
      33,    63,    64,    65,    66,    67,    68,    69,    70,    71,
      72,    73,    74,    75,    76,    77,    78,    79,    80,    81,
      82,    83,    84,    85,    86,    87,    88,    89,    90,    91,
      92,    93,    94,    95,    96,    97,    98,    99,   100,   101,
     102,   103,   104,   105,   106,   107,   108,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   155,    -1,    -1,    -1,    -1,
      -1,   161,   162,   163,   164,   165,   166,   167,   168,   169,
     170,   171,   172,   173,   174,   175,   176,   177,   178,   179,
     180,   181,   182,   183,   184,   185,   186,   187,   188,   189,
     190,   191,   192,   193,   194,   195,   196,   197,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   206,   207,   208,   209,
     210,   211,   155,    -1,    -1,    -1,    -1,    -1,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
     193,   194,   195,   196,   197,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   206,   207,   208,   209,   210,   211,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    80,    81,    82,    83,
      84,    85,    86,    87,    88,    89,    90,    91,    92,    93,
      94,    95,    96,    97,    98,    99,   100,   101,   102,   103,
     104,   105,   106,   107,   108,   109,   110,   111,   112,   113,
     114,   115,   116,    -1,    -1,   119,   120,    -1,    -1,   123,
     124,    -1,    -1,   127,   128,    -1,    -1,    -1,    -1,   133,
     134,   135,    -1,   137,   138,   139,   140,   141,   142,   143,
     144,   145,   146,   147,   148,    -1,   150,    -1,   152,   153,
      -1,   155,   156,   157,   158,   159,   160,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   171,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   209
  };

  const short
  parser::yystos_[] =
  {
       0,     1,   213,   214,   215,     0,    12,    13,   216,     3,
       4,   218,     6,   117,   118,   130,   132,   136,   151,   208,
     210,   217,   219,   220,   221,   222,   223,   231,   259,   260,
     265,   266,   296,    10,    12,    13,    26,    33,   155,   161,
     162,   163,   164,   165,   166,   167,   168,   169,   170,   171,
     172,   173,   174,   175,   176,   177,   178,   179,   180,   181,
     182,   183,   184,   185,   186,   187,   188,   189,   190,   191,
     192,   193,   194,   195,   196,   197,   206,   207,   208,   209,
     210,   211,   229,   230,   321,   322,   325,   328,     3,   261,
     325,   267,   324,   325,     6,     7,     6,     7,   218,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    80,    81,    82,    83,
      84,    85,    86,    87,    88,    89,    90,    91,    92,    93,
      94,    95,    96,    97,    98,    99,   100,   101,   102,   103,
     104,   105,   106,   107,   108,   109,   110,   111,   112,   113,
     114,   115,   116,   119,   120,   123,   124,   127,   128,   133,
     134,   135,   137,   138,   139,   140,   141,   142,   143,   144,
     145,   146,   147,   148,   150,   152,   153,   155,   156,   157,
     158,   159,   160,   171,   209,   224,   225,   227,   232,   235,
     236,   239,   241,   245,   246,   249,   250,   251,   252,   253,
     255,   256,   258,   270,   271,   273,   274,   275,   276,   278,
     279,   280,   281,   282,   283,   284,   285,   286,   287,   288,
     289,   292,   295,   297,   298,   300,   301,   302,   303,   304,
     305,   306,   332,   333,   338,   339,   340,   341,   342,   343,
     344,   345,   346,   347,   348,   349,   350,   351,   352,   353,
     354,   355,   356,   357,   358,   360,   361,   362,   363,   364,
     365,   366,   367,   368,   369,   370,   371,   372,   373,   374,
     375,   376,   377,   378,   379,   380,   381,   382,   383,   384,
     385,   386,   387,   388,   389,   390,   391,   392,   393,   394,
     321,   321,   321,   321,   321,    10,    10,    10,    10,    10,
      10,    10,    10,    10,    10,    10,   323,    10,    10,    10,
      10,    10,    10,    10,    10,    10,    10,    10,    10,    10,
      10,    10,    10,    10,    10,    10,    10,    10,    10,    10,
      10,    10,    10,    12,    13,    14,    15,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    25,    27,    28,    29,
      30,    31,    32,     3,   209,     3,   209,     3,     8,    46,
      47,    48,    49,    50,    51,    52,   176,   180,   316,   321,
     396,   397,   398,   399,   400,   401,   402,   403,   404,   407,
     408,   409,    56,    57,   396,   397,   396,   397,   309,   325,
      26,    48,    60,    61,    62,   319,   321,   405,   406,   396,
     397,   176,   398,    54,    55,    56,    57,   176,   180,   398,
     407,   411,   407,   411,    56,   319,   405,   319,   405,     8,
     398,     8,   359,   395,   398,     8,   398,     8,    54,    55,
      56,    57,   359,   395,   408,   409,   412,   413,   414,   396,
     397,    53,    54,    55,    56,   410,   410,   309,    26,   405,
     407,   407,   407,   407,   316,   396,   397,   309,   407,   407,
     407,   316,   396,   397,   407,   396,   397,   240,   324,   128,
     129,   160,   254,   218,   328,   310,   311,   317,   321,   322,
     328,   314,   315,   322,   328,   109,   324,   312,   313,   320,
     322,   328,   225,   227,   229,   236,   293,   294,   328,   328,
     328,   131,   159,   334,   209,   242,   307,   308,   322,   326,
     328,   307,   290,   209,   247,   334,   228,   324,   334,   209,
     257,   325,   254,   328,   226,   233,    35,    36,    37,    38,
      39,    40,    41,    42,    43,    44,    45,    93,   125,   126,
     149,   154,   238,    35,   125,   126,   238,     7,    11,   230,
     325,   325,   325,   325,   230,   328,   321,   325,   328,   328,
     325,    10,   325,   325,   325,   325,   321,   328,   321,   325,
     321,   325,   325,   325,   198,   199,   200,   201,   202,   203,
     204,   205,   328,   335,   328,   335,    11,   328,   329,   328,
     328,   330,   328,   328,   328,   328,   328,   328,   328,   325,
     321,   321,   321,   321,   321,   321,   321,   321,   321,   321,
     321,   321,   321,   321,   321,   321,   321,   321,   321,   321,
     321,   262,   268,   263,    56,    10,    10,     5,     5,     5,
       5,   405,     5,    10,    10,    10,     5,     5,    56,     5,
     180,   319,   321,   400,     5,     5,     5,    56,     5,    54,
      55,    56,    58,    59,     5,     5,     5,     5,     5,     5,
       5,     5,     5,     5,     5,     5,     5,     5,     5,   321,
       5,     5,   237,     5,   237,     8,     5,     5,   237,     5,
     237,     5,   218,   328,     5,   207,   243,   244,     5,   237,
     237,   229,   291,   243,   248,   328,   209,   328,   325,   209,
     234,   325,   272,   324,   325,   328,   272,   272,   325,   325,
     325,   328,   325,   338,    11,     5,   327,   327,   327,     5,
      11,    11,    11,   327,    11,     5,   327,   230,     5,   327,
       5,     5,    11,    11,    11,     5,    11,     5,   327,   327,
      11,    11,    11,    11,     5,    11,     5,   331,    11,     5,
      11,    11,     5,     5,     5,    11,   327,    11,     5,     3,
     218,     9,    53,    54,    55,    56,    54,    55,    56,   316,
     407,   411,   316,   407,   319,   319,   319,     9,     8,    10,
       9,    12,     9,   398,   398,   359,   395,     9,     8,     9,
       9,    12,    13,     9,     9,   319,   319,    57,   319,    56,
     319,   398,    57,   398,   316,   407,   316,   359,   395,   407,
     412,   398,   407,   407,   325,     5,   299,   325,   311,   315,
     240,   277,   316,   313,   294,   325,     5,   209,   244,   308,
       5,   237,     5,     5,     5,   207,   325,    11,    11,    11,
     325,    11,   325,    11,    11,   325,    11,   325,   325,   325,
     325,    11,    11,   328,   328,     5,   328,   328,   328,   325,
      11,   264,   325,   269,    11,    11,    11,    11,    11,    11,
      11,     5,    56,   400,     5,    56,     9,     9,    12,    13,
     318,   328,     5,   237,     9,     5,   237,     5,   335,   229,
     335,   335,   328,   327,    11,   327,   327,   327,   327,   327,
      11,   326,   328,    11,    11,     5,     5,    11,     3,     5,
     218,   398,     9,     9,   398,     9,   321,   321,   325,     5,
     237,   316,   325,     8,   336,   336,   336,    11,    11,    11,
      11,    11,    11,   328,   324,   263,   325,   277,   324,   337,
     337,   337,    11,    11,   218,     5,   237,     9,     5,   325,
     109,   165,     8,     8,   240,   324,     9,     9
  };

  const short
  parser::yyr1_[] =
  {
       0,   212,   213,   214,   214,   215,   214,   216,   216,   216,
     217,   217,   218,   218,   219,   219,   219,   219,   219,   219,
     219,   219,   220,   221,   222,   223,   223,   223,   223,   224,
     226,   225,   228,   227,   229,   229,   229,   230,   230,   231,
     231,   231,   231,   231,   231,   231,   233,   232,   234,   234,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   236,
     236,   236,   236,   236,   236,   237,   237,   238,   238,   238,
     238,   238,   238,   238,   238,   238,   238,   239,   240,   240,
     242,   241,   243,   243,   244,   245,   247,   246,   248,   248,
     249,   250,   251,   252,   253,   254,   254,   254,   254,   255,
     255,   255,   255,   256,   257,   257,   258,   258,   259,   261,
     262,   260,   263,   264,   264,   264,   265,   267,   268,   266,
     269,   270,   271,   272,   272,   273,   274,   275,   276,   276,
     276,   276,   277,   277,   278,   278,   279,   279,   280,   280,
     281,   282,   283,   283,   283,   283,   284,   285,   286,   287,
     288,   290,   289,   291,   291,   292,   293,   293,   294,   295,
     296,   297,   297,   297,   298,   299,   299,   300,   300,   301,
     302,   303,   304,   305,   306,   306,   307,   307,   308,   308,
     309,   310,   310,   311,   311,   312,   312,   313,   313,   314,
     314,   315,   315,   316,   317,   318,   318,   319,   320,   321,
     321,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   323,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   322,   322,   322,   322,   322,   322,   322,   322,
     322,   322,   324,   325,   326,   327,   327,   328,   328,   328,
     328,   328,   328,   328,   328,   328,   328,   328,   329,   329,
     330,   330,   331,   331,   331,   332,   333,   334,   334,   334,
     335,   335,   335,   335,   335,   335,   335,   335,   336,   336,
     337,   337,   337,   338,   338,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   339,   339,   339,   339,   339,   339,   339,   339,   339,
     339,   340,   340,   341,   341,   341,   341,   342,   342,   343,
     344,   344,   345,   346,   346,   347,   347,   348,   349,   349,
     350,   351,   352,   353,   353,   354,   354,   354,   355,   355,
     356,   356,   357,   357,   358,   358,   358,   358,   359,   359,
     360,   360,   360,   360,   360,   360,   360,   360,   361,   361,
     362,   362,   363,   363,   364,   365,   366,   366,   367,   367,
     367,   367,   367,   368,   368,   369,   370,   370,   371,   372,
     373,   374,   374,   375,   376,   377,   378,   379,   380,   381,
     382,   383,   384,   385,   385,   386,   387,   388,   389,   390,
     391,   391,   392,   392,   393,   394,   394,   395,   396,   396,
     397,   397,   398,   398,   399,   399,   400,   400,   401,   401,
     402,   402,   403,   403,   404,   404,   405,   405,   406,   406,
     406,   406,   407,   407,   408,   408,   408,   408,   408,   408,
     408,   409,   410,   410,   410,   410,   411,   411,   411,   411,
     412,   412,   412,   412,   413,   413,   414,   414
  };

  const signed char
  parser::yyr2_[] =
  {
       0,     2,     1,     0,     3,     0,     3,     0,     1,     1,
       2,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     3,     3,     2,     1,     2,     2,     2,     1,
       0,     3,     0,     3,     1,     1,     1,     1,     1,     0,
       1,     1,     2,     2,     2,     2,     0,     3,     0,     2,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     0,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     2,     1,     3,
       0,     3,     1,     2,     1,     1,     0,     3,     0,     1,
       1,     1,     1,     2,     2,     0,     2,     2,     2,     3,
       5,     3,     5,     2,     0,     1,     7,     1,     5,     0,
       0,     9,     0,     1,     3,     5,     3,     0,     0,     7,
       0,     2,     1,     0,     1,     1,     1,     1,     2,     5,
       6,     8,     1,     3,     1,     3,     1,     3,     1,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     0,     4,     1,     3,     3,     1,     3,     1,     2,
       4,     2,     4,     6,     5,     1,     3,     2,     4,     2,
       1,     2,     1,     3,     1,     3,     1,     3,     1,     1,
       1,     1,     3,     1,     1,     1,     3,     1,     1,     1,
       3,     1,     1,     1,     1,     2,     2,     1,     1,     1,
       1,     1,     1,     2,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     2,     2,     2,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     0,     5,
       5,     5,     5,     7,     7,     7,     7,     7,     5,     5,
       5,     5,     5,     5,     7,     6,     6,     6,     4,     4,
       4,     3,     1,     1,     1,     0,     2,     1,     8,     6,
       6,     3,     4,     4,     4,     8,     4,     4,     1,     3,
       0,     2,     0,     3,     3,     7,     7,     0,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     0,     3,
       0,     6,     6,     1,     3,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     2,     2,     2,     2,     4,     4,     2,     2,     4,
       2,     4,     1,     2,     2,     1,     2,     1,     2,     2,
       1,     1,     1,     2,     2,     2,     4,     2,     2,     4,
       6,     6,     6,     6,     4,     4,     4,     4,     3,     5,
       1,     1,     1,     1,     1,     1,     1,     1,     5,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     4,     4,     4,     1,     2,     2,     2,     2,
       4,     1,     2,     1,     2,     1,     2,     1,     2,     1,
       2,     1,     2,     2,     2,     1,     4,     2,     2,     2,
       1,     2,     2,     2,     2,     2,     2,     3,     1,     3,
       1,     3,     1,     4,     1,     4,     1,     4,     1,     4,
       1,     4,     1,     4,     1,     4,     1,     2,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       3,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       3,     3,     1,     1,     3,     4,     3,     4
  };




#if YYDEBUG
  const short
  parser::yyrline_[] =
  {
       0,   402,   402,   405,   406,   408,   408,   418,   419,   424,
     434,   435,   438,   438,   445,   446,   447,   448,   449,   450,
     452,   453,   457,   468,   486,   505,   506,   507,   508,   512,
     518,   518,   527,   527,   538,   541,   544,   550,   553,   559,
     560,   563,   566,   569,   572,   576,   583,   583,   592,   595,
     602,   603,   604,   605,   606,   607,   608,   609,   610,   611,
     612,   613,   614,   615,   616,   617,   618,   619,   620,   621,
     622,   623,   624,   625,   626,   627,   628,   629,   630,   631,
     632,   633,   634,   635,   636,   637,   638,   639,   640,   644,
     645,   646,   647,   648,   649,   652,   652,   655,   658,   661,
     664,   667,   670,   673,   676,   679,   682,   688,   694,   703,
     722,   722,   729,   730,   734,   740,   746,   746,   754,   757,
     761,   767,   773,   779,   785,   791,   794,   797,   800,   806,
     813,   820,   824,   831,   841,   844,   848,   851,   857,   864,
     866,   864,   875,   881,   886,   891,   899,   906,   908,   906,
     917,   923,   929,   935,   938,   942,   948,   954,   960,   963,
     966,   972,   981,   984,   991,   994,   998,  1001,  1005,  1008,
    1012,  1019,  1026,  1030,  1034,  1038,  1045,  1054,  1063,  1072,
    1079,  1086,  1086,  1096,  1099,  1105,  1108,  1109,  1113,  1119,
    1125,  1133,  1138,  1143,  1151,  1157,  1160,  1167,  1170,  1176,
    1182,  1188,  1195,  1200,  1203,  1207,  1214,  1215,  1219,  1222,
    1229,  1239,  1240,  1244,  1247,  1254,  1255,  1259,  1262,  1269,
    1270,  1274,  1277,  1284,  1291,  1298,  1302,  1309,  1316,  1323,
    1326,  1333,  1336,  1339,  1342,  1345,  1348,  1351,  1354,  1357,
    1360,  1363,  1366,  1369,  1372,  1375,  1378,  1381,  1384,  1387,
    1390,  1393,  1396,  1399,  1402,  1405,  1408,  1411,  1414,  1417,
    1420,  1423,  1426,  1430,  1433,  1436,  1439,  1442,  1445,  1445,
    1451,  1454,  1457,  1460,  1463,  1466,  1469,  1472,  1475,  1478,
    1481,  1484,  1487,  1490,  1493,  1496,  1499,  1504,  1509,  1512,
    1515,  1518,  1524,  1532,  1538,  1544,  1547,  1557,  1560,  1566,
    1572,  1578,  1581,  1584,  1588,  1592,  1595,  1598,  1618,  1621,
    1628,  1629,  1636,  1637,  1641,  1648,  1654,  1661,  1664,  1667,
    1673,  1676,  1679,  1682,  1685,  1688,  1691,  1694,  1700,  1703,
    1713,  1718,  1723,  1732,  1733,  1737,  1738,  1739,  1740,  1741,
    1742,  1743,  1744,  1745,  1746,  1747,  1748,  1749,  1750,  1751,
    1752,  1753,  1754,  1755,  1756,  1757,  1758,  1759,  1760,  1761,
    1762,  1763,  1764,  1765,  1766,  1767,  1768,  1769,  1770,  1771,
    1772,  1773,  1774,  1775,  1776,  1777,  1778,  1779,  1780,  1781,
    1782,  1786,  1790,  1796,  1800,  1803,  1806,  1813,  1817,  1823,
    1830,  1834,  1841,  1847,  1851,  1857,  1860,  1866,  1872,  1875,
    1881,  1887,  1893,  1899,  1902,  1908,  1912,  1916,  1922,  1926,
    1933,  1936,  1942,  1945,  1951,  1962,  1973,  1976,  1982,  1983,
    1991,  1992,  1993,  1994,  1995,  1996,  1997,  1998,  2002,  2006,
    2013,  2016,  2023,  2027,  2034,  2041,  2047,  2051,  2060,  2064,
    2067,  2071,  2074,  2081,  2085,  2094,  2100,  2104,  2110,  2116,
    2122,  2129,  2132,  2138,  2144,  2151,  2157,  2164,  2170,  2177,
    2183,  2190,  2196,  2206,  2210,  2216,  2222,  2229,  2236,  2243,
    2250,  2254,  2261,  2265,  2271,  2278,  2282,  2290,  2296,  2297,
    2303,  2306,  2314,  2315,  2319,  2320,  2324,  2325,  2329,  2330,
    2334,  2335,  2339,  2340,  2344,  2345,  2349,  2350,  2356,  2359,
    2362,  2365,  2370,  2370,  2373,  2376,  2379,  2382,  2385,  2388,
    2391,  2397,  2403,  2406,  2409,  2412,  2418,  2421,  2424,  2427,
    2433,  2436,  2439,  2442,  2448,  2449,  2453,  2454
  };

  void
  parser::yy_stack_print_ () const
  {
    *yycdebug_ << "Stack now";
    for (stack_type::const_iterator
           i = yystack_.begin (),
           i_end = yystack_.end ();
         i != i_end; ++i)
      *yycdebug_ << ' ' << int (i->state);
    *yycdebug_ << '\n';
  }

  void
  parser::yy_reduce_print_ (int yyrule) const
  {
    int yylno = yyrline_[yyrule];
    int yynrhs = yyr2_[yyrule];
    // Print the symbols being reduced, and their result.
    *yycdebug_ << "Reducing stack by rule " << yyrule - 1
               << " (line " << yylno << "):\n";
    // The symbols being reduced.
    for (int yyi = 0; yyi < yynrhs; yyi++)
      YY_SYMBOL_PRINT ("   $" << yyi + 1 << " =",
                       yystack_[(yynrhs) - (yyi + 1)]);
  }
#endif // YYDEBUG


} // yy
#line 5971 "src/asm/parser.cpp"

#line 2457 "src/asm/parser.y"


/******************** Semantic actions ********************/

void yy::parser::error(std::string const &str) {
	::error("%s\n", str.c_str());
}

static uint32_t strToNum(std::vector<int32_t> const &s) {
	uint32_t length = s.size();

	if (length == 1) {
		// The string is a single character with a single value,
		// which can be used directly as a number.
		return static_cast<uint32_t>(s[0]);
	}

	warning(WARNING_OBSOLETE, "Treating multi-unit strings as numbers is deprecated\n");

	for (int32_t v : s) {
		if (!checkNBit(v, 8, "All character units"))
			break;
	}

	uint32_t r = 0;

	for (uint32_t i = length < 4 ? 0 : length - 4; i < length; i++) {
		r <<= 8;
		r |= static_cast<uint8_t>(s[i]);
	}

	return r;
}

static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName) {
	error("%s: Invalid UTF-8 byte 0x%02hhX\n", functionName, byte);
}

static size_t strlenUTF8(std::string const &str) {
	char const *ptr = str.c_str();
	size_t len = 0;
	uint32_t state = 0;

	for (uint32_t codep = 0; *ptr; ptr++) {
		uint8_t byte = *ptr;

		switch (decode(&state, &codep, byte)) {
		case 1:
			errorInvalidUTF8Byte(byte, "STRLEN");
			state = 0;
			// fallthrough
		case 0:
			len++;
			break;
		}
	}

	// Check for partial code point.
	if (state != 0)
		error("STRLEN: Incomplete UTF-8 character\n");

	return len;
}

static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len) {
	char const *ptr = str.c_str();
	size_t index = 0;
	uint32_t state = 0;
	uint32_t codep = 0;
	uint32_t curPos = 1; // RGBASM strings are 1-indexed!

	// Advance to starting position in source string.
	while (ptr[index] && curPos < pos) {
		switch (decode(&state, &codep, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curPos++;
			break;
		}
		index++;
	}

	// A position 1 past the end of the string is allowed, but will trigger the
	// "Length too big" warning below if the length is nonzero.
	if (!ptr[index] && pos > curPos)
		warning(
		    WARNING_BUILTIN_ARG, "STRSUB: Position %" PRIu32 " is past the end of the string\n", pos
		);

	size_t startIndex = index;
	uint32_t curLen = 0;

	// Compute the result length in bytes.
	while (ptr[index] && curLen < len) {
		switch (decode(&state, &codep, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curLen++;
			break;
		}
		index++;
	}

	if (curLen < len)
		warning(WARNING_BUILTIN_ARG, "STRSUB: Length too big: %" PRIu32 "\n", len);

	// Check for partial code point.
	if (state != 0)
		error("STRSUB: Incomplete UTF-8 character\n");

	return std::string(ptr + startIndex, ptr + index);
}

static size_t charlenUTF8(std::string const &str) {
	std::string_view view = str;
	size_t len;

	for (len = 0; charmap_ConvertNext(view, nullptr); len++)
		;

	return len;
}

static std::string charsubUTF8(std::string const &str, uint32_t pos) {
	std::string_view view = str;
	size_t charLen = 1;

	// Advance to starting position in source string.
	for (uint32_t curPos = 1; charLen && curPos < pos; curPos++)
		charLen = charmap_ConvertNext(view, nullptr);

	std::string_view start = view;

	if (!charmap_ConvertNext(view, nullptr))
		warning(
		    WARNING_BUILTIN_ARG,
		    "CHARSUB: Position %" PRIu32 " is past the end of the string\n",
		    pos
		);

	start = start.substr(0, start.length() - view.length());
	return std::string(start);
}

static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName) {
	// STRSUB and CHARSUB adjust negative `pos` arguments the same way,
	// such that position -1 is the last character of a string.
	if (pos < 0)
		pos += len + 1;
	if (pos < 1) {
		warning(WARNING_BUILTIN_ARG, "%s: Position starts at 1\n", functionName);
		pos = 1;
	}
	return static_cast<uint32_t>(pos);
}

static std::string strrpl(std::string_view str, std::string const &old, std::string const &rep) {
	if (old.empty()) {
		warning(WARNING_EMPTY_STRRPL, "STRRPL: Cannot replace an empty string\n");
		return std::string(str);
	}

	std::string rpl;

	while (!str.empty()) {
		auto pos = str.find(old);
		if (pos == str.npos) {
			rpl.append(str);
			break;
		}
		rpl.append(str, 0, pos);
		rpl.append(rep);
		str.remove_prefix(pos + old.size());
	}

	return rpl;
}

static std::string strfmt(
    std::string const &spec, std::vector<Either<uint32_t, std::string>> const &args
) {
	std::string str;
	size_t argIndex = 0;

	for (size_t i = 0; spec[i] != '\0'; ++i) {
		int c = spec[i];

		if (c != '%') {
			str += c;
			continue;
		}

		c = spec[++i];

		if (c == '%') {
			str += c;
			continue;
		}

		FormatSpec fmt{};

		while (c != '\0') {
			fmt.useCharacter(c);
			if (fmt.isFinished())
				break;
			c = spec[++i];
		}

		if (fmt.isEmpty()) {
			error("STRFMT: Illegal '%%' at end of format string\n");
			str += '%';
			break;
		}

		if (!fmt.isValid()) {
			error("STRFMT: Invalid format spec for argument %zu\n", argIndex + 1);
			str += '%';
		} else if (argIndex >= args.size()) {
			// Will warn after formatting is done.
			str += '%';
		} else if (args[argIndex].holds<uint32_t>()) {
			fmt.appendNumber(str, args[argIndex].get<uint32_t>());
		} else {
			fmt.appendString(str, args[argIndex].get<std::string>());
		}

		argIndex++;
	}

	if (argIndex < args.size())
		error("STRFMT: %zu unformatted argument(s)\n", args.size() - argIndex);
	else if (argIndex > args.size())
		error(
		    "STRFMT: Not enough arguments for format spec, got: %zu, need: %zu\n",
		    args.size(),
		    argIndex
		);

	return str;
}

static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue) {
	Expression oldExpr, constExpr, newExpr;
	int32_t newValue;

	oldExpr.makeSymbol(symName);
	constExpr.makeNumber(constValue);
	newExpr.makeBinaryOp(op, std::move(oldExpr), constExpr);
	newValue = newExpr.getConstVal();
	sym_AddVar(symName, newValue);
}

static void failAssert(AssertionType type) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed\n");
	case ASSERT_ERROR:
		error("Assertion failed\n");
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed\n");
		break;
	}
}

static void failAssertMsg(AssertionType type, std::string const &message) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed: %s\n", message.c_str());
	case ASSERT_ERROR:
		error("Assertion failed: %s\n", message.c_str());
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed: %s\n", message.c_str());
		break;
	}
}
